@@ -31,12 +31,16 @@
 
 #include "config.h"
 
+#define SOFIA_EXTEND_AUTH_CLIENT 1
+
 #include <sofia-sip/su.h>
 #include <sofia-sip/su_md5.h>
 
+#include "sofia-sip/auth_common.h"
 #include "sofia-sip/auth_client.h"
 #include "sofia-sip/auth_client_plugin.h"
 
+#include <sofia-sip/msg_types.h>
 #include <sofia-sip/msg_header.h>
 
 #include <sofia-sip/auth_digest.h>
@@ -65,15 +69,17 @@ static int ca_challenge(auth_client_t *ca,
 			char const *scheme,
 			char const *realm);
 
+static int ca_info(auth_client_t *ca,
+		   msg_auth_info_t const *ai,
+		   msg_hclass_t *credential_class);
+
 static int ca_credentials(auth_client_t *ca, 
 			  char const *scheme,
 			  char const *realm, 
 			  char const *user,
 			  char const *pass);
 
-static int ca_clear_credentials(auth_client_t *ca, 
-				char const *scheme,
-				char const *realm);
+static int ca_clear_credentials(auth_client_t *ca);
 
 
 /** Initialize authenticators.
@@ -81,10 +87,10 @@ static int ca_clear_credentials(auth_client_t *ca,
  * The function auc_challenge() merges the challenge @a ch to the list of
  * authenticators @a auc_list.  
  *
- * @param auc_list [in/out] list of authenticators to be updated
- * @param home     [in/out] memory home used for allocating authenticators
- * @param ch       [in] challenge to be processed
- * @param crcl     [in] credential class
+ * @param[in,out] auc_list  list of authenticators to be updated
+ * @param[in,out] home      memory home used for allocating authenticators
+ * @param[in] ch        challenge to be processed
+ * @param[in] crcl      credential class
  * 
  * @retval 1 when challenge was updated
  * @retval 0 when there was no new challenges
@@ -172,11 +178,97 @@ int ca_challenge(auth_client_t *ca,
     return -1;
 
   if (!ca->ca_credential_class)
-    stale = 1, ca->ca_credential_class = credential_class;
+    stale = 2, ca->ca_credential_class = credential_class;
 
-  return stale ? 2 : 1;
+  return stale > 1 ? 2 : 1;
 }
 
+/** Store authentication info to authenticators.
+ *
+ * The function auc_info() feeds the authentication data from the @b
+ * Authentication-Info header @a info to the list of authenticators @a
+ * auc_list.
+ *
+ * @param[in,out] auc_list  list of authenticators to be updated
+ * @param[in] info        info header to be processed
+ * @param[in] credential_class      corresponding credential class
+ *
+ * The authentication info can be in either @AuthenticationInfo or in
+ * @ProxyAuthenticationInfo headers. If the header is @AuthenticationInfo,
+ * the @a credential_class should be #sip_authorization_class or
+ * #http_authorization_class. Likewise, If the header is
+ * @ProxyAuthenticationInfo, the @a credential_class should be
+ * #sip_proxy_authorization_class or #http_proxy_authorization_class.
+
+ * The authentication into header usually contains next nonce or mutual
+ * authentication information. Currently, only the nextnonce parameter is
+ * processed.
+ *
+ * @bug
+ * The result can be quite unexpected if there are more than one
+ * authenticator with the given type (specified by @a crcl). In principle,
+ * SIP allows more than one challenge for a single request.
+ *
+ * @retval number of challenges to updated
+ * @retval 0 when there was no challenge to update
+ * @retval -1 upon an error
+ *
+ * @NEW_1_12_5
+ */
+int auc_info(auth_client_t **auc_list,
+	     msg_auth_info_t const *info,
+	     msg_hclass_t *credential_class)
+{
+  auth_client_t *ca;
+  int retval = 0;
+
+  /* Go through each challenge in Authenticate or Proxy-Authenticate headers */
+
+  /* Update matching authenticator */
+  for (ca = *auc_list; ca; ca = ca->ca_next) {
+    int updated = ca_info(ca, info, credential_class);
+    if (updated < 0)
+      return -1;
+    if (updated >= 1)
+      retval = 1;		/* Updated authenticator */
+  }
+
+  return retval;
+}
+
+/** Update authentication client with authentication info. 
+ *
+ * @retval -1 upon an error
+ * @retval 0 when challenge did not match
+ * @retval 1 when challenge did match but was not updated
+ * @retval 2 when challenge did match and updated client
+ */
+static
+int ca_info(auth_client_t *ca, 
+	    msg_auth_info_t const *info,
+	    msg_hclass_t *credential_class)
+{
+  assert(ca); assert(info);
+
+  if (!ca || !info)
+    return -1;
+
+  if (!ca->ca_credential_class)
+    return 0;
+
+  if (ca->ca_credential_class != credential_class)
+    return 0;
+
+  if (!ca->ca_auc
+      || (size_t)ca->ca_auc->auc_plugin_size <= 
+         offsetof(auth_client_plugin_t, auc_info)
+      || !ca->ca_auc->auc_info)
+    return 0;
+
+  return ca->ca_auc->auc_info(ca, info);
+}
+
+
 /**Feed authentication data to the authenticator.
  *
  * The function auc_credentials() is used to provide the authenticators in
@@ -189,9 +281,9 @@ int ca_challenge(auth_client_t *ca,
  *
  * @todo The authentication data format sucks.
  *
- * @param auc_list [in/out] list of authenticators 
- * @param home     [in/out] memory home used for allocations
- * @param data     [in]     colon-separated authentication data
+ * @param[in,out] auc_list  list of authenticators 
+ * @param[in,out] home      memory home used for allocations
+ * @param[in] data          colon-separated authentication data
  * 
  * @retval 0 when successful
  * @retval -1 upon an error
@@ -242,14 +334,14 @@ int auc_credentials(auth_client_t **auc_list, su_home_t *home,
  *
  * @todo The authentication data format sucks.
  *
- * @param auc_list [in/out] list of authenticators 
- * @param scheme   [in]     scheme to use (NULL, if any)
- * @param realm    [in]     realm to use (NULL, if any)
- * @param user     [in]     username 
- * @param pass     [in]     password
+ * @param[in,out] auc_list  list of authenticators 
+ * @param[in] scheme        scheme to use (NULL, if any)
+ * @param[in] realm         realm to use (NULL, if any)
+ * @param[in] user          username 
+ * @param[in] pass          password
  * 
- * @retval number of matching clients
- * @retval 0 when no matching client was found
+ * @retval number of updated clients
+ * @retval 0 when no client was updated
  * @retval -1 upon an error
  */
 int auc_all_credentials(auth_client_t **auc_list, 
@@ -283,6 +375,9 @@ int ca_credentials(auth_client_t *ca,
 		   char const *user,
 		   char const *pass)
 {
+  char *new_user, *new_pass;
+  char *old_user, *old_pass;
+
   assert(ca);
 
   if (!ca || !ca->ca_scheme || !ca->ca_realm)
@@ -292,12 +387,24 @@ int ca_credentials(auth_client_t *ca,
       (realm != NULL && strcmp(realm, ca->ca_realm)))
     return -1;
 
-  ca->ca_user = su_strdup(ca->ca_home, user);
-  ca->ca_pass = su_strdup(ca->ca_home, pass);
+  old_user = ca->ca_user, old_pass = ca->ca_pass;
 
-  if (!ca->ca_user || !ca->ca_pass)
+  if (str0cmp(user, old_user) == 0 && str0cmp(pass, old_pass) == 0)
+    return 0;
+
+  new_user = su_strdup(ca->ca_home, user);
+  new_pass = su_strdup(ca->ca_home, pass);
+
+  if (!new_user || !new_pass)
     return -1;
 
+  ca->ca_user = new_user, ca->ca_pass = new_pass;
+  if (AUTH_CLIENT_IS_EXTENDED(ca))
+    ca->ca_clear = 0;
+
+  su_free(ca->ca_home, old_user);
+  su_free(ca->ca_home, old_pass);
+
   return 1;
 }
 
@@ -324,12 +431,15 @@ int auc_copy_credentials(auth_client_t **dst,
       char *u, *p;
       if (!ca->ca_user || !ca->ca_pass)
 	continue;
+      if (AUTH_CLIENT_IS_EXTENDED(ca) && ca->ca_clear)
+	continue;
       if (!ca->ca_scheme[0] || strcmp(ca->ca_scheme, d->ca_scheme))
 	continue;
       if (!ca->ca_realm[0] || strcmp(ca->ca_realm, d->ca_realm))
 	continue;
 
-      if (d->ca_user && strcmp(d->ca_user, ca->ca_user) == 0 &&
+      if (!(AUTH_CLIENT_IS_EXTENDED(d) && d->ca_clear) &&
+	  d->ca_user && strcmp(d->ca_user, ca->ca_user) == 0 &&
 	  d->ca_pass && strcmp(d->ca_pass, ca->ca_pass) == 0) {
 	retval++;
 	break;
@@ -343,6 +453,9 @@ int auc_copy_credentials(auth_client_t **dst,
       if (d->ca_user) su_free(d->ca_home, (void *)d->ca_user);
       if (d->ca_pass) su_free(d->ca_home, (void *)d->ca_pass);
       d->ca_user = u, d->ca_pass = p;
+      if (AUTH_CLIENT_IS_EXTENDED(d))
+	d->ca_clear = 0;
+
       retval++;
       break;
     }
@@ -357,21 +470,32 @@ int auc_copy_credentials(auth_client_t **dst,
  * The function auc_clear_credentials() is used to remove the credentials
  * from the authenticators.
  *
- * @param auc_list [in/out] list of authenticators 
- * @param scheme   [in] scheme (if non-null, remove only matching credentials) 
- * @param realm    [in] realm (if non-null, remove only matching credentials)
+ * @param[in,out] auc_list  list of authenticators 
+ * @param[in] scheme    scheme (if non-null, remove only matching credentials) 
+ * @param[in] realm     realm (if non-null, remove only matching credentials)
  *
  * @retval 0 when successful
  * @retval -1 upon an error
  */
 int auc_clear_credentials(auth_client_t **auc_list, 
-			 char const *scheme,
-			 char const *realm)
+			  char const *scheme,
+			  char const *realm)
 {
   int retval = 0;
+  int match;
 
   for (; *auc_list; auc_list = &(*auc_list)->ca_next) {
-    int match = ca_clear_credentials(*auc_list, scheme, realm);
+    auth_client_t *ca = *auc_list;
+
+    if (!AUTH_CLIENT_IS_EXTENDED(ca))
+      continue;
+
+    if ((scheme != NULL && strcasecmp(scheme, ca->ca_scheme)) ||
+	(realm != NULL && strcmp(realm, ca->ca_realm)))
+      continue;
+
+    match = ca->ca_auc->auc_clear(*auc_list);
+
     if (match < 0) {
       retval = -1;
       break;
@@ -384,21 +508,39 @@ int auc_clear_credentials(auth_client_t **auc_list,
 }
 
 static
-int ca_clear_credentials(auth_client_t *ca, 
-			 char const *scheme,
-			 char const *realm)
+int ca_clear_credentials(auth_client_t *ca)
 {
-  assert(ca);
+  assert(ca); assert(ca->ca_home->suh_size >= (int)(sizeof *ca));
 
-  if (!ca || !ca->ca_scheme || !ca->ca_realm)
+  if (!ca)
     return -1;
 
-  if ((scheme != NULL && strcasecmp(scheme, ca->ca_scheme)) ||
-      (realm != NULL && strcmp(realm, ca->ca_realm)))
-    return -1;
+  ca->ca_clear = 1;
 
-  su_free(ca->ca_home, (void *)ca->ca_user), ca->ca_user = NULL;
-  su_free(ca->ca_home, (void *)ca->ca_pass), ca->ca_pass = NULL;
+  return 1;
+}
+
+/** Check if we have all required credentials.
+ * 
+ * @retval 1 when authorization can proceed
+ * @retval 0 when there is not enough credentials
+ *
+ * @NEW_1_12_5
+ */
+int auc_has_authorization(auth_client_t **auc_list)
+{
+  auth_client_t const *ca;
+
+  if (auc_list == NULL)
+    return 0;
+
+  /* Make sure every challenge has credentials */
+  for (ca = *auc_list; ca; ca = ca->ca_next) {
+    if (!ca->ca_user || !ca->ca_pass || !ca->ca_credential_class)
+      return 0;
+    if (AUTH_CLIENT_IS_EXTENDED(ca) && ca->ca_clear)
+      return 0;
+  }
 
   return 1;
 }
@@ -409,12 +551,12 @@ int ca_clear_credentials(auth_client_t *ca,
  * headers to a request. The authentication headers will contain the
  * credentials generated by the list of authenticators.
  *
- * @param auc_list [in/out] list of authenticators 
- * @param msg      [out]    message to be authenticated
- * @param pub      [out]    headers of the message
- * @param method   [in]     request method
- * @param url      [in]     request URI
- * @param body     [in]     message body (NULL if empty)
+ * @param[in,out] auc_list  list of authenticators 
+ * @param[out] msg          message to be authenticated
+ * @param[out] pub          headers of the message
+ * @param[in] method        request method
+ * @param[in] url           request URI
+ * @param[in] body          message body (NULL if empty)
  * 
  * @retval 1 when successful
  * @retval 0 when there is not enough credentials
@@ -431,15 +573,12 @@ int auc_authorization(auth_client_t **auc_list, msg_t *msg, msg_pub_t *pub,
   if (auc_list == NULL || msg == NULL)
     return -1;
 
+  if (!auc_has_authorization(auc_list))
+    return 0;
+
   if (pub == NULL)
     pub = msg_object(msg);
 
-  /* Make sure every challenge has credentials */
-  for (ca = *auc_list; ca; ca = ca->ca_next) {
-    if (!ca->ca_user || !ca->ca_pass || !ca->ca_credential_class)
-      return 0;
-  }
-
   /* Remove existing credentials */
   for (ca = *auc_list; ca; ca = ca->ca_next) {
     msg_header_t **hh = msg_hclass_offset(mc, pub, ca->ca_credential_class);
@@ -458,8 +597,11 @@ int auc_authorization(auth_client_t **auc_list, msg_t *msg, msg_pub_t *pub,
     if (!ca->ca_auc)
       continue;
 
-    if (ca->ca_auc->auc_authorize(ca, home, method, url, body, &h) < 0
-	|| msg_header_insert(msg, pub, h) < 0)
+    if (ca->ca_auc->auc_authorize(ca, home, method, url, body, &h) < 0)
+      return -1;
+    if (h == NULL)
+      continue;
+    if (msg_header_insert(msg, pub, h) < 0)
       return -1;
   }
 
@@ -472,12 +614,12 @@ int auc_authorization(auth_client_t **auc_list, msg_t *msg, msg_pub_t *pub,
  * authentication headers for a request. The list of authentication headers
  * will contain the credentials generated by the list of authenticators.
  *
- * @param auc_list [in/out] list of authenticators 
- * @param home     [in]     memory home used to allocate headers
- * @param method   [in]     request method
- * @param url      [in]     request URI
- * @param body     [in]     message body (NULL if empty)
- * @param return_headers [out] authorization headers
+ * @param[in] auc_list      list of authenticators 
+ * @param[in] home          memory home used to allocate headers
+ * @param[in] method        request method
+ * @param[in] url           request URI
+ * @param[in] body          message body (NULL if empty)
+ * @param[out] return_headers  authorization headers return value
  * 
  * @retval 1 when successful
  * @retval 0 when there is not enough credentials
@@ -493,12 +635,10 @@ int auc_authorization_headers(auth_client_t **auc_list,
   auth_client_t *ca;
 
   /* Make sure every challenge has credentials */
-  for (ca = *auc_list; ca; ca = ca->ca_next) {
-    if (!ca->ca_user || !ca->ca_pass || !ca->ca_credential_class)
-      return 0;
-  }
+  if (!auc_has_authorization(auc_list))
+    return 0;
 
-  /* Insert new credentials */
+  /* Create new credential headers */
   for (; *auc_list; auc_list = &(*auc_list)->ca_next) {
     msg_header_t *h = NULL;
 
@@ -529,13 +669,15 @@ static int auc_basic_authorization(auth_client_t *ca,
 				   msg_payload_t const *body,
 				   msg_header_t **);
 
-const auth_client_plugin_t ca_basic_plugin = 
+static const auth_client_plugin_t ca_basic_plugin = 
 { 
-  sizeof ca_basic_plugin,
-  sizeof (auth_client_t),
-  "Basic",
-  NULL,
-  auc_basic_authorization
+  /* auc_plugin_size: */ sizeof ca_basic_plugin,
+  /* auc_size: */        sizeof (auth_client_t),
+  /* auc_name: */       "Basic",
+  /* auc_challenge: */   NULL,
+  /* auc_authorize: */   auc_basic_authorization,
+  /* auc_info: */        NULL,
+  /* auc_clear: */       ca_clear_credentials
 };
 
 /**Create a basic authorization header.
@@ -572,6 +714,9 @@ int auc_basic_authorization(auth_client_t *ca,
   if (user == NULL || pass == NULL)
     return -1;
 
+  if (AUTH_CLIENT_IS_EXTENDED(ca) && ca->ca_clear)
+    return 0;
+
   ulen = strlen(user), plen = strlen(pass), uplen = ulen + 1 + plen;
   b64len = BASE64_SIZE(uplen);
   basiclen = strlen("Basic ") + b64len;
@@ -625,17 +770,25 @@ static int auc_digest_authorization(auth_client_t *ca,
 				    url_t const *url, 
 				    msg_payload_t const *body,
 				    msg_header_t **);
+static int auc_digest_info(auth_client_t *ca, 
+			   msg_auth_info_t const *info);
 
 static const auth_client_plugin_t ca_digest_plugin = 
 { 
-  sizeof ca_digest_plugin,
-  sizeof (auth_digest_client_t),
-  "Digest", 
-  auc_digest_challenge,
-  auc_digest_authorization
+  /* auc_plugin_size: */ sizeof ca_digest_plugin,
+  /* auc_size: */        sizeof (auth_digest_client_t),
+  /* auc_name: */       "Digest", 
+  /* auc_challenge: */   auc_digest_challenge,
+  /* auc_authorize: */   auc_digest_authorization,
+  /* auc_info: */        auc_digest_info,
+  /* auc_clear: */       ca_clear_credentials
 };
 
 /** Store a digest authorization challenge.
+ *
+ * @retval 2 if credentials need to be (re)sent
+ * @retval 1 if challenge was updated
+ * @retval -1 upon an error
  */
 static int auc_digest_challenge(auth_client_t *ca, msg_auth_t const *ch)
 {
@@ -653,17 +806,18 @@ static int auc_digest_challenge(auth_client_t *ca, msg_auth_t const *ch)
   if (ac->ac_qop && !ac->ac_auth && !ac->ac_auth_int)
     goto error;
 
-  stale = ac->ac_stale || str0cmp(ac->ac_nonce, cda->cda_ac->ac_nonce);
+  stale = ac->ac_stale || cda->cda_ac->ac_nonce == NULL;
 
   if (ac->ac_qop && (cda->cda_cnonce == NULL || ac->ac_stale)) {
     su_guid_t guid[1];
     char *cnonce;
+    size_t b64len = BASE64_MINSIZE(sizeof(guid)) + 1;
     if (cda->cda_cnonce != NULL)
       /* Free the old one if we are updating after stale=true */
       su_free(home, (void *)cda->cda_cnonce);
     su_guid_generate(guid);
-    cda->cda_cnonce = cnonce = su_alloc(home, BASE64_SIZE(sizeof(guid)) + 1);
-    base64_e(cnonce, BASE64_SIZE(sizeof(guid)) + 1, guid, sizeof(guid));
+    cda->cda_cnonce = cnonce = su_alloc(home, b64len);
+    base64_e(cnonce, b64len, guid, sizeof(guid));
     cda->cda_ncount = 0;
   }
 
@@ -678,6 +832,25 @@ static int auc_digest_challenge(auth_client_t *ca, msg_auth_t const *ch)
   return -1;
 }
 
+static int auc_digest_info(auth_client_t *ca,
+			   msg_auth_info_t const *info)
+{
+  auth_digest_client_t *cda = (auth_digest_client_t *)ca;
+  su_home_t *home = ca->ca_home;
+  char const *nextnonce = NULL;
+  issize_t n;
+
+  n = auth_get_params(home, info->ai_params,
+		      "nextnonce=", &nextnonce,
+		      NULL);
+
+  if (n <= 0)
+    return n;
+
+  cda->cda_ac->ac_nonce = nextnonce;
+
+  return 1;
+}
 
 /**Create a digest authorization header.
  *
@@ -688,22 +861,11 @@ static int auc_digest_challenge(auth_client_t *ca, msg_auth_t const *ch)
  * sip_authorization_class or sip_proxy_authorization_class, as well as
  * http_authorization_class or http_proxy_authorization_class.
  *
- * @param home 	  memory home used to allocate memory for the new header
- * @param hc   	  header class for the header to be created
- * @param user 	  user name
- * @param pass 	  password
- * @param ac      challenge structure
- * @param cnonce  client nonce
- * @param nc      client nonce count 
- * @param method  request method
- * @param uri     request uri
- * @param data    message body
- * @param dlen    length of message body
- *
- * @return
- * Returns a pointer to newly created authorization header, or NULL upon an
- * error.
+ * @retval 1 when authorization headers has been created  
+ * @retval 0 when there is no credentials
+ * @retval -1 upon an error
  */
+static
 int auc_digest_authorization(auth_client_t *ca, 
 			     su_home_t *home,
 			     char const *method, 
@@ -727,6 +889,9 @@ int auc_digest_authorization(auth_client_t *ca,
   auth_response_t ar[1] = {{ 0 }};
   char ncount[17];
 
+  if (!user || !pass || (AUTH_CLIENT_IS_EXTENDED(ca) && ca->ca_clear))
+    return 0;
+
   ar->ar_size = sizeof(ar);
   ar->ar_username = user;
   ar->ar_realm = ac->ac_realm;
@@ -879,9 +1044,9 @@ void ca_destroy(su_home_t *home, auth_client_t *ca)
  * headers to a SIP request. The authentication headers will contain the
  * credentials generated by the list of authenticators.
  *
- * @param auc_list [in/out] list of authenticators 
- * @param msg      [in/out] message to be authenticated
- * @param sip      [in/out] sip headers of the message
+ * @param[in,out] auc_list  list of authenticators 
+ * @param[in,out] msg       message to be authenticated
+ * @param[in,out] sip       sip headers of the message
  * 
  * @retval 1 when successful
  * @retval 0 when there is not enough credentials
