@@ -76,7 +76,7 @@ static void msg_insert_chain(msg_t *msg, msg_pub_t *pub, int prepend,
 static void msg_insert_here_in_chain(msg_t *msg,
 				     msg_header_t **prev,
 				     msg_header_t *h);
-static inline msg_header_t *msg_chain_remove(msg_t *msg, msg_header_t *h);
+su_inline msg_header_t *msg_chain_remove(msg_t *msg, msg_header_t *h);
 
 #ifndef NDEBUG
 static int msg_chain_loop(msg_header_t const *h);
@@ -122,12 +122,12 @@ void msg_set_streaming(msg_t *msg, enum msg_streaming_status what)
 /** Test if header is not in the chain */
 #define msg_header_is_removed(h) ((h)->sh_prev == NULL)
 
-static inline int msg_is_request(msg_header_t const *h)
+su_inline int msg_is_request(msg_header_t const *h)
 {
   return h->sh_class->hc_hash == msg_request_hash;
 }
 
-static inline int msg_is_status(msg_header_t const *h)
+su_inline int msg_is_status(msg_header_t const *h)
 {
   return h->sh_class->hc_hash == msg_status_hash;
 }
@@ -253,7 +253,7 @@ usize_t msg_buf_size(msg_t const *msg)
     return 0;
 }
 
-static inline
+su_inline
 void msg_buf_used(msg_t *msg, usize_t used)
 {
   msg->m_size += used;
@@ -812,10 +812,10 @@ int msg_unref_external(msg_t *msg, msg_buffer_t *b)
 /* ====================================================================== */
 /* Parsing messages */
 
-static inline int extract_incomplete_chunks(msg_t *, int eos);
+su_inline int extract_incomplete_chunks(msg_t *, int eos);
 static issize_t extract_first(msg_t *, msg_pub_t *,
 			    char b[], isize_t bsiz, int eos);
-static inline issize_t extract_next(msg_t *, msg_pub_t *, char *, isize_t bsiz, 
+su_inline issize_t extract_next(msg_t *, msg_pub_t *, char *, isize_t bsiz, 
 				  int eos, int copy);
 static issize_t extract_header(msg_t *, msg_pub_t*,
 			     char b[], isize_t bsiz, int eos, int copy);
@@ -823,14 +823,14 @@ static msg_header_t *header_parse(msg_t *, msg_pub_t *, msg_href_t const *,
 				  char s[], isize_t slen, int copy_buffer);
 static msg_header_t *error_header_parse(msg_t *msg, msg_pub_t *mo,
 					msg_href_t const *hr);
-static inline issize_t
+su_inline issize_t
 extract_trailers(msg_t *msg, msg_pub_t *mo,
 		 char *b, isize_t bsiz, int eos, int copy);
 
 /** Calculate length of line ending (0, 1 or 2). @internal */
 #define CRLF_TEST(b) ((b)[0] == '\r' ? ((b)[1] == '\n') + 1 : (b)[0] =='\n')
 
-static inline void
+su_inline void
 append_parsed(msg_t *msg, msg_pub_t *mo, msg_href_t const *hr, msg_header_t *h,
 	      int always_into_chain);
 
@@ -991,7 +991,7 @@ issize_t extract_first(msg_t *msg, msg_pub_t *mo, char b[], isize_t bsiz, int eo
 }
 
 /* Extract header or message body */
-static inline issize_t
+su_inline issize_t
 extract_next(msg_t *msg, msg_pub_t *mo, char *b, isize_t bsiz, 
 	     int eos, int copy)
 {
@@ -1190,9 +1190,8 @@ msg_header_t *error_header_parse(msg_t *msg, msg_pub_t *mo,
 /** Complete this header field and parse next header field. 
  *
  * This function completes parsing a multi-field header like @Accept,
- * @Contact, @Via or @Warning. It updates the shortcuts to header parameters
- * (see msg_header_update_params()) and then scans for the next header field.
- * If one is found, it calls the parsing function recursively.
+ * @Contact, @Via or @Warning. It scans for the next header field and 
+ * if one is found, it calls the parsing function recursively.
  *
  * @param home 	 memory home used ot allocate 
  *             	 new header structures and parameter lists
@@ -1214,38 +1213,26 @@ issize_t msg_parse_next_field(su_home_t *home, msg_header_t *prev,
   msg_header_t *h;
   char *end = s + slen;
 
-  if (hc->hc_update && hc->hc_params) {
-    /* Update shortcuts to parameters */
-    msg_param_t p, v, *params;
-
-    params = *(msg_param_t **)((char *)prev + hc->hc_params);
-    if (params) {
-      for (p = *params; p; p = *++params) {
-	size_t n = strcspn(p, "=");
-	v = p + n + (p[n] == '=');
-	if (hc->hc_update(prev->sh_common, p, n, v) < 0)
-	  return -1;
-      }
-    }
-  }
-
   if (*s && *s != ',')
     return -1;
 
+  if (msg_header_update_params(prev->sh_common, 0) < 0)
+    return -1;
+
   while (*s == ',') /* Skip comma and following whitespace */
     *s = '\0', s += span_lws(s + 1) + 1;
 
   if (*s == 0)
     return 0;
   
-  h = msg_header_alloc(home, prev->sh_class, 0);
+  h = msg_header_alloc(home, hc, 0);
   if (!h)
     return -1;
 
   prev->sh_succ = h, h->sh_prev = &prev->sh_succ;
   prev->sh_next = h;
 
-  return prev->sh_class->hc_parse(home, h, s, end - s);
+  return hc->hc_parse(home, h, s, end - s);
 }
 
 /** Decode a message header. */
@@ -1315,7 +1302,7 @@ issize_t msg_extract_separator(msg_t *msg, msg_pub_t *mo,
   return l;
 }
 
-static inline msg_header_t **msg_chain_tail(msg_t const *msg);
+su_inline msg_header_t **msg_chain_tail(msg_t const *msg);
 
 /** Extract a message body of @a body_len bytes.
   */
@@ -1472,7 +1459,7 @@ issize_t msg_extract_payload(msg_t *msg, msg_pub_t *mo,
 
 /** Extract incomplete chunks.
  */
-static inline
+su_inline
 int extract_incomplete_chunks(msg_t *msg, int eos)
 {
   msg_payload_t *chunk;
@@ -1508,7 +1495,7 @@ int extract_incomplete_chunks(msg_t *msg, int eos)
 }
 
 /* Extract trailers */
-static inline issize_t
+su_inline issize_t
 extract_trailers(msg_t *msg, msg_pub_t *mo,
 		 char *b, isize_t bsiz, int eos, int copy)
 {
@@ -1524,7 +1511,7 @@ extract_trailers(msg_t *msg, msg_pub_t *mo,
 /* Preparing (printing/encoding) a message structure for sending */
 
 /* Internal prototypes */
-static inline size_t
+su_inline size_t
 msg_header_name_e(char b[], size_t bsiz, msg_header_t const *h, int flags);
 static size_t msg_header_prepare(msg_mclass_t const *, int flags,
 				 msg_header_t *h, msg_header_t **return_next,
@@ -1744,7 +1731,7 @@ issize_t msg_header_e(char b[], isize_t bsiz, msg_header_t const *h, int flags)
 }
 
 /** Encode header name */
-static inline
+su_inline
 size_t
 msg_header_name_e(char b[], size_t bsiz, msg_header_t const *h, int flags)
 {
@@ -1867,7 +1854,7 @@ char *msg_as_string(su_home_t *home, msg_t *msg, msg_pub_t *pub, int flags,
 /* ====================================================================== */
 /* Handling header chain */
 
-static inline void serialize_first(msg_t *msg, msg_header_t *h);
+su_inline void serialize_first(msg_t *msg, msg_header_t *h);
 static msg_header_t **serialize_one(msg_t *msg, msg_header_t *h,
 				    msg_header_t **prev);
 
@@ -1877,13 +1864,13 @@ msg_header_t **msg_chain_head(msg_t const *msg)
   return msg ? (msg_header_t **)&msg->m_chain : NULL;
 }
 
-static inline msg_header_t **_msg_chain_head(msg_t const *msg)
+su_inline msg_header_t **_msg_chain_head(msg_t const *msg)
 {
   return msg ? (msg_header_t **)&msg->m_chain : NULL;
 }
 
 /** Return tail of the fragment chain */
-static inline msg_header_t **msg_chain_tail(msg_t const *msg)
+su_inline msg_header_t **msg_chain_tail(msg_t const *msg)
 {
   return msg ? msg->m_tail : NULL;
 }
@@ -1990,7 +1977,7 @@ int msg_serialize(msg_t *msg, msg_pub_t *pub)
   return 0;
 }
 
-static inline
+su_inline
 void serialize_first(msg_t *msg, msg_header_t *h)
 {
   if (msg_header_is_removed(h)) {
@@ -2215,7 +2202,7 @@ void msg_insert_here_in_chain(msg_t *msg,
  *
  * @return The pointer to the header just removed.
  */
-static inline
+su_inline
 msg_header_t *msg_chain_remove(msg_t *msg, msg_header_t *h)
 {
   if (h) {
@@ -2490,7 +2477,7 @@ msg_hclass_offset(msg_mclass_t const *mc, msg_pub_t const *mo, msg_hclass_t *hc)
 }
 
 /** Append a parsed header object into the message structure */
-static inline void
+su_inline void
 append_parsed(msg_t *msg, msg_pub_t *mo, msg_href_t const *hr, msg_header_t *h,
 	      int always_into_chain)
 {
@@ -2524,6 +2511,10 @@ append_parsed(msg_t *msg, msg_pub_t *mo, msg_href_t const *hr, msg_header_t *h,
   *hh = h;
 }
 
+static int _msg_header_add_list_items(msg_t *msg, 
+				      msg_header_t **hh,
+				      msg_header_t const *src);
+
 /**Duplicate and add a (list of) header(s) to the message.
  *
  * The function @c msg_header_add_dup() duplicates and adds a (list of)
@@ -2588,24 +2579,8 @@ int msg_header_add_dup(msg_t *msg,
       hh = &h->sh_next;
     }
     else {
-      /* Add list items */
-      msg_header_t *h = *hh;
-      msg_param_t **d, **s;
-
-      d = msg_header_params(h->sh_common); assert(d);
-      s = msg_header_params(src->sh_common);
-
-      if (!s || !*s)
-	return 0;
-
-      msg_fragment_clear(h->sh_common);
-
-      /* Remove empty headers */
-      for (hh = &h->sh_next; *hh; *hh = (*hh)->sh_next)
-	msg_chain_remove(msg, *hh);
-
-      if (msg_params_join(msg_home(msg), d, *s, 2 /* prune case */, 1) < 0)
-	return -1;
+      if (_msg_header_add_list_items(msg, hh, src) < 0)
+	break;
     }
   }
 
@@ -2661,25 +2636,8 @@ int _msg_header_add_dup_as(msg_t *msg,
   if (hh == NULL)
     return -1;
 
-  if (*hh && hc->hc_kind == msg_kind_list) {
-    /* Add list items */
-    msg_header_t *h = *hh;
-    msg_param_t **d, **s;
-
-    d = msg_header_params(h->sh_common); assert(d);
-    s = msg_header_params(src->sh_common);
-
-    if (!s || !*s)
-      return 0;
-
-    msg_fragment_clear(h->sh_common);
-
-    /* Remove empty headers */
-    for (hh = &h->sh_next; *hh; *hh = (*hh)->sh_next)
-      msg_chain_remove(msg, *hh);
-
-    return msg_params_join(msg_home(msg), d, *s, 2 /* prune case */, 1);
-  }
+  if (*hh && hc->hc_kind == msg_kind_list)
+    return _msg_header_add_list_items(msg, hh, src);
 
   if (!(h = msg_header_dup_as(msg_home(msg), hc, src)))
     return -1;
@@ -2687,6 +2645,30 @@ int _msg_header_add_dup_as(msg_t *msg,
   return msg_header_add(msg, pub, hh, h);
 }
 
+/* Add list items */
+static int _msg_header_add_list_items(msg_t *msg, 
+				      msg_header_t **hh,
+				      msg_header_t const *src)
+{
+  msg_header_t *h = *hh;
+  msg_param_t **s = msg_header_params(src->sh_common);
+
+  if (!s || !*s)
+    return 0;
+  
+  msg_fragment_clear(h->sh_common);
+  
+  /* Remove empty headers */
+  for (hh = &h->sh_next; *hh; *hh = (*hh)->sh_next)
+    msg_chain_remove(msg, *hh);
+  
+  if (msg_header_join_items(msg_home(msg), h->sh_common, src->sh_common, 1)
+      < 0)
+    return -1;
+  
+  return 0;
+}
+
 /** Parse a string as a given header field and add result to the message. */
 int msg_header_add_make(msg_t *msg,
 			msg_pub_t *pub,
