@@ -630,20 +630,20 @@ int readfile(FILE *f, void **contents)
   /* Read in whole (binary!) file */
   char *buffer = NULL;
   long size;
-  int len = -1;
+  size_t len;
   
   /* Read whole file in */
   if (fseek(f, 0, SEEK_END) < 0 ||
       (size = ftell(f)) < 0 ||
       fseek(f, 0, SEEK_SET) < 0 ||
-      (long)(len = size) != size) {
+      (long)(len = (size_t)size) != size) {
     fprintf(stderr, "%s: unable to determine file size (%s)\n", 
 	    __func__, strerror(errno));
     return -1;
   }
 
   if (!(buffer = malloc(len + 2)) ||
-      fread(buffer, 1, len, f) != (size_t)len) {
+      fread(buffer, 1, len, f) != len) {
     fprintf(stderr, "%s: unable to read file (%s)\n", __func__, strerror(errno));
     if (buffer)
       free(buffer);
@@ -654,7 +654,7 @@ int readfile(FILE *f, void **contents)
 
   *contents = buffer;
 
-  return len;
+  return (int)len;
 }
 
 #if HAVE_DIRENT_H
@@ -663,6 +663,8 @@ int readfile(FILE *f, void **contents)
 
 static int test_bad_messages(agent_t *ag)
 {
+  BEGIN();
+
 #if HAVE_DIRENT_H
   DIR *dir;
   struct dirent *d;
@@ -687,13 +689,10 @@ static int test_bad_messages(agent_t *ag)
   if (dir == NULL) {
     fprintf(stderr, "test_nta: cannot find sip torture messages\n"); 
     fprintf(stderr, "test_nta: tried %s\n", name); 
-    return 0;
   }
 
   offset = strlen(name);
 
-  BEGIN();
-
   TEST_1(ag->ag_default_leg = nta_leg_tcreate(ag->ag_agent, 
 					      leg_callback_500,
 					      ag,
@@ -713,14 +712,14 @@ static int test_bad_messages(agent_t *ag)
   
   TEST(su_getaddrinfo(host, port, hints, &ai), 0); TEST_1(ai);
   s = su_socket(ai->ai_family, ai->ai_socktype, ai->ai_protocol); TEST_1(s != -1);
-  memset(su, 0, sulen = sizeof su); 
+  memset(su, 0, sulen = ai->ai_addrlen); 
   su->su_len = sizeof su; su->su_family = ai->ai_family;
   TEST_1(bind(s, &su->su_sa, sulen) == 0);
   TEST_1(getsockname(s, &su->su_sa, &sulen) == 0);
   sprintf(via, "v: SIP/2.0/UDP is.invalid:%u\r\n", ntohs(su->su_port));
   vlen = strlen(via);
 
-  for (d = readdir(dir); d; d = readdir(dir)) {
+  for (d = dir ? readdir(dir) : NULL; d; d = readdir(dir)) {
     size_t len = strlen(d->d_name);
     FILE *f;
     int blen, n;
@@ -733,6 +732,7 @@ static int test_bad_messages(agent_t *ag)
     strncpy(name + offset, d->d_name, PATH_MAX - offset);
     TEST_1(f = fopen(name, "rb"));
     TEST_1((blen = readfile(f, &buffer)) > 0);
+    fclose(f);
     r = buffer;
 
     if (strncmp(r, "JUNK ", 5) == 0) {
@@ -751,6 +751,12 @@ static int test_bad_messages(agent_t *ag)
     su_root_step(ag->ag_root, 1);
   }
 
+  TEST_SIZE(su_sendto(s, "\r\n\r\n", 4, 0, (void *)ai->ai_addr, ai->ai_addrlen), 4);
+
+  su_root_step(ag->ag_root, 1);
+
+  TEST_SIZE(su_sendto(s, "", 0, 0, ai->ai_addr, ai->ai_addrlen), 0);
+
   su_close(s);
 
   for (i = 0; i < 20; i++)
@@ -758,12 +764,12 @@ static int test_bad_messages(agent_t *ag)
 
   nta_leg_destroy(ag->ag_default_leg), ag->ag_default_leg = NULL;
 
-  closedir(dir);
+  if (dir)
+    closedir(dir);
+
+#endif /* HAVE_DIRENT_H */
 
   END();
-#else
-  return 0;
-#endif /* HAVE_DIRENT_H */
 }
 
 static unsigned char const code[] = 
@@ -771,12 +777,12 @@ static unsigned char const code[] =
 
 #include <sofia-sip/su_uniqueid.h>
 
-sip_payload_t *test_payload(su_home_t *home, int size)
+sip_payload_t *test_payload(su_home_t *home, size_t size)
 {
-  sip_payload_t *pl = sip_payload_create(home, NULL, size);
+  sip_payload_t *pl = sip_payload_create(home, NULL, (isize_t)size);
 
   if (pl) {
-    int i;
+    size_t i;
     char *data = (char *)pl->pl_data;
     
     for (i = 0; i < size; i++) {
@@ -928,7 +934,7 @@ int test_tports(agent_t *ag)
   if (tcp_comp) {
     url_t url[1];
     sip_payload_t *pl;
-    unsigned size = 1024;
+    size_t size = 1024;
 
     *url = *ag->ag_aliases->m_url;
     url->url_user = "alice";
@@ -973,7 +979,7 @@ int test_tports(agent_t *ag)
   if (tcp) {
     url_t url[1];
     sip_payload_t *pl;
-    unsigned size = 512 * 1024;
+    usize_t size = 512 * 1024;
 
     *url = *ag->ag_aliases->m_url;
     url->url_user = "alice";
@@ -1009,7 +1015,7 @@ int test_tports(agent_t *ag)
   if (tcp) {
     url_t url[1];
     sip_payload_t *pl;
-    unsigned size = 512 * 1024;
+    usize_t size = 512 * 1024;
 
     *url = *ag->ag_aliases->m_url;
     url->url_user = "alice";
@@ -1043,7 +1049,7 @@ int test_tports(agent_t *ag)
   {
     url_t url[1];
     sip_payload_t *pl;
-    unsigned size = 2 * 1024;
+    usize_t size = 2 * 1024;
 
     *url = *ag->ag_aliases->m_url;
     url->url_user = "alice";
@@ -1082,7 +1088,7 @@ int test_tports(agent_t *ag)
   if (v_udp_only) {
     url_t url[1];
     sip_payload_t *pl;
-    unsigned size = 2 * 1024;
+    usize_t size = 2 * 1024;
 
     *url = *ag->ag_aliases->m_url;
     url->url_user = "alice";
@@ -1125,7 +1131,7 @@ int test_tports(agent_t *ag)
   if (udp) {
     url_t url[1];
     sip_payload_t *pl;
-    unsigned size = 2 * 1024;
+    usize_t size = 2 * 1024;
 
     *url = *ag->ag_aliases->m_url;
     url->url_user = "alice";
@@ -1206,7 +1212,7 @@ int test_tports(agent_t *ag)
   if (sctp) {
     url_t url[1];
     sip_payload_t *pl;
-    unsigned size = 16 * 1024;
+    usize_t size = 16 * 1024;
 
     *url = *ag->ag_aliases->m_url;
     url->url_user = "alice";
@@ -1243,7 +1249,7 @@ int test_tports(agent_t *ag)
   if (tcp) {
     url_t url[1];
     sip_payload_t *pl;
-    unsigned size = 128 * 1024;
+    usize_t size = 128 * 1024;
 
     nta_agent_set_params(ag->ag_agent, 
 			 NTATAG_MAXSIZE(65536),
@@ -1328,6 +1334,105 @@ int test_tports(agent_t *ag)
   END();
 }
 
+int leg_callback_destroy(agent_t *ag,
+			 nta_leg_t *leg,
+			 nta_incoming_t *irq,
+			 sip_t const *sip)
+{
+  if (tstflags & tst_verbatim) {
+    printf("%s: %s: %s " URL_PRINT_FORMAT " %s\n",
+	   name, __func__, sip->sip_request->rq_method_name, 
+	   URL_PRINT_ARGS(sip->sip_request->rq_url),
+	   sip->sip_request->rq_version);
+  }
+
+  ag->ag_latest_leg = leg;
+
+  nta_incoming_destroy(irq);
+
+  return 0;
+}
+
+int leg_callback_save(agent_t *ag,
+		      nta_leg_t *leg,
+		      nta_incoming_t *irq,
+		      sip_t const *sip)
+{
+  if (tstflags & tst_verbatim) {
+    printf("%s: %s: %s " URL_PRINT_FORMAT " %s\n",
+	   name, __func__, sip->sip_request->rq_method_name, 
+	   URL_PRINT_ARGS(sip->sip_request->rq_url),
+	   sip->sip_request->rq_version);
+  }
+
+  ag->ag_latest_leg = leg;
+  ag->ag_irq = irq;
+  ag->ag_status = 1000;
+
+  return 0;
+}
+
+
+int test_destroy_incoming(agent_t *ag)
+{
+  BEGIN();
+
+  url_t url[1];
+
+  *url = *ag->ag_contact->m_url;
+
+  /* Test 3.1
+   * Check that when a incoming request is destroyed in callback, 
+   * a 500 response is sent
+   */
+  ag->ag_expect_leg = ag->ag_default_leg;
+  nta_leg_bind(ag->ag_default_leg, leg_callback_destroy, ag);
+
+  TEST_1(ag->ag_orq = 
+	 nta_outgoing_tcreate(ag->ag_default_leg, 
+			      outgoing_callback, ag,
+			      ag->ag_obp,
+			      SIP_METHOD_MESSAGE,
+			      (url_string_t *)url,
+			      SIPTAG_SUBJECT_STR("Test 3.1"),
+			      SIPTAG_FROM(ag->ag_alice),
+			      SIPTAG_TO(ag->ag_bob),
+			      TAG_END()));
+
+  nta_test_run(ag);
+  TEST(ag->ag_status, 500);
+  TEST_P(ag->ag_orq, NULL);
+  TEST_P(ag->ag_latest_leg, ag->ag_default_leg);
+
+  /* Test 3.1
+   * Check that when a incoming request is destroyed, a 500 response is sent
+   */
+  nta_leg_bind(ag->ag_default_leg, leg_callback_save, ag);
+
+  TEST_1(ag->ag_orq = 
+	 nta_outgoing_tcreate(ag->ag_default_leg, 
+			      outgoing_callback, ag,
+			      ag->ag_obp,
+			      SIP_METHOD_MESSAGE,
+			      (url_string_t *)url,
+			      SIPTAG_SUBJECT_STR("Test 3.1"),
+			      SIPTAG_FROM(ag->ag_alice),
+			      SIPTAG_TO(ag->ag_bob),
+			      TAG_END()));
+
+  nta_test_run(ag);
+  TEST(ag->ag_status, 1000);
+  TEST_1(ag->ag_irq);
+  TEST_1(ag->ag_orq);
+  TEST_P(ag->ag_latest_leg, ag->ag_default_leg);
+  nta_incoming_destroy(ag->ag_irq), ag->ag_irq = NULL;
+  nta_test_run(ag);
+  TEST(ag->ag_status, 500);
+  TEST_P(ag->ag_orq, NULL);
+
+  END();
+}
+
 int test_resolv(agent_t *ag, char const *resolv_conf)
 {
   int udp = 0, tcp = 0, sctp = 0, tls = 0;
@@ -1340,6 +1445,8 @@ int test_resolv(agent_t *ag, char const *resolv_conf)
 
   BEGIN();
 
+  nta_leg_bind(ag->ag_default_leg, leg_callback_200, ag);
+
   nta_agent_set_params(ag->ag_agent, 
 		       NTATAG_SIP_T1(8 * 25), 
 		       NTATAG_SIP_T1X64(64 * 25), 
@@ -1698,7 +1805,7 @@ int test_resolv(agent_t *ag, char const *resolv_conf)
   if (tcp_comp) {
     url_t url[1];
     sip_payload_t *pl;
-    unsigned size = 1024;
+    usize_t size = 1024;
 
     *url = *ag->ag_aliases->m_url;
     url->url_user = "alice";
@@ -1736,7 +1843,7 @@ int test_resolv(agent_t *ag, char const *resolv_conf)
   if (tcp) {
     url_t url[1];
     sip_payload_t *pl;
-    unsigned size = 512 * 1024;
+    usize_t size = 512 * 1024;
 
     *url = *ag->ag_aliases->m_url;
     url->url_user = "alice";
@@ -1777,7 +1884,7 @@ int test_resolv(agent_t *ag, char const *resolv_conf)
   if (tcp) {
     url_t url[1];
     sip_payload_t *pl;
-    unsigned size = 512 * 1024;
+    usize_t size = 512 * 1024;
 
     *url = *ag->ag_aliases->m_url;
     url->url_user = "alice";
@@ -1811,7 +1918,7 @@ int test_resolv(agent_t *ag, char const *resolv_conf)
   {
     url_t url[1];
     sip_payload_t *pl;
-    unsigned size = 2 * 1024;
+    usize_t size = 2 * 1024;
 
     *url = *ag->ag_aliases->m_url;
     url->url_user = "alice";
@@ -1850,7 +1957,7 @@ int test_resolv(agent_t *ag, char const *resolv_conf)
   if (udp) {
     url_t url[1];
     sip_payload_t *pl;
-    unsigned size = 2 * 1024;
+    usize_t size = 2 * 1024;
 
     *url = *ag->ag_aliases->m_url;
     url->url_user = "alice";
@@ -1930,7 +2037,7 @@ int test_resolv(agent_t *ag, char const *resolv_conf)
   if (sctp) {
     url_t url[1];
     sip_payload_t *pl;
-    unsigned size = 16 * 1024;
+    usize_t size = 16 * 1024;
 
     *url = *ag->ag_aliases->m_url;
     url->url_user = "alice";
@@ -1967,7 +2074,7 @@ int test_resolv(agent_t *ag, char const *resolv_conf)
   if (tcp) {
     url_t url[1];
     sip_payload_t *pl;
-    unsigned size = 128 * 1024;
+    usize_t size = 128 * 1024;
 
     nta_agent_set_params(ag->ag_agent, 
 			 NTATAG_MAXSIZE(65536),
@@ -2064,6 +2171,8 @@ int test_routing(agent_t *ag)
   *url = *ag->ag_aliases->m_url;
   url->url_user = "bob";
 
+  nta_leg_bind(ag->ag_default_leg, leg_callback_200, ag);
+
   nta_agent_set_params(ag->ag_agent, 
 		       NTATAG_MAXSIZE(2 * 1024 * 1024),
 		       TAG_END());
@@ -3254,6 +3363,7 @@ char const nta_test_usage[] =
   "usage: %s OPTIONS\n"
   "where OPTIONS are\n"
   "   -v | --verbose    be verbose\n"
+  "   -a | --abort      abort() on error\n"
   "   -q | --quiet      be quiet\n"
   "   -1                quit on first error\n"
   "   -l level          set logging level (0 by default)\n"
@@ -3265,10 +3375,10 @@ char const nta_test_usage[] =
 #endif
   ;
 
-void usage(void)
+void usage(int exitcode)
 {
   fprintf(stderr, nta_test_usage, name);
-  exit(1);
+  exit(exitcode);
 }
 
 int main(int argc, char *argv[])
@@ -3281,6 +3391,8 @@ int main(int argc, char *argv[])
   for (i = 1; argv[i]; i++) {
     if (strcmp(argv[i], "-v") == 0 || strcmp(argv[i], "--verbose") == 0)
       tstflags |= tst_verbatim;
+    else if (strcmp(argv[i], "-a") == 0 || strcmp(argv[i], "--abort") == 0)
+      tstflags |= tst_abort;
     else if (strcmp(argv[i], "-q") == 0 || strcmp(argv[i], "--quiet") == 0)
       tstflags &= ~tst_verbatim;
     else if (strcmp(argv[i], "-1") == 0)
@@ -3297,7 +3409,7 @@ int main(int argc, char *argv[])
 	level = 3, rest = "";
 
       if (rest == NULL || *rest)
-	usage();
+	usage(1);
       
       su_log_set_level(nta_log, level);
       su_log_set_level(tport_log, level);
@@ -3308,7 +3420,7 @@ int main(int argc, char *argv[])
       else if (argv[i + 1])
 	ag->ag_obp = (url_string_t *)(argv[++i]);
       else
-	usage();
+	usage(1);
     }
     else if (strncmp(argv[i], "-m", 2) == 0) {
       if (argv[i][2])
@@ -3316,7 +3428,7 @@ int main(int argc, char *argv[])
       else if (argv[i + 1])
 	ag->ag_m = argv[++i];
       else
-	usage();
+	usage(1);
     }
     else if (strcmp(argv[i], "--attach") == 0) {
       o_attach = 1;
@@ -3331,14 +3443,14 @@ int main(int argc, char *argv[])
       break;
     }
     else
-      usage();
+      usage(1);
   }
 
   if (o_attach) {
-    char line[10];
+    char line[10], *got;
     printf("nua_test: pid %u\n", getpid());
     printf("<Press RETURN to continue>\n");
-    fgets(line, sizeof line, stdin);
+    got = fgets(line, sizeof line, stdin); (void)got;
   }
 #if HAVE_ALARM
   else if (o_alarm) {
@@ -3364,6 +3476,7 @@ int main(int argc, char *argv[])
     retval |= test_bad_messages(ag); SINGLE_FAILURE_CHECK();
     retval |= test_reinit(ag); SINGLE_FAILURE_CHECK();
     retval |= test_tports(ag); SINGLE_FAILURE_CHECK();
+    retval |= test_destroy_incoming(ag); SINGLE_FAILURE_CHECK();
     retval |= test_resolv(ag, argv[i]); SINGLE_FAILURE_CHECK();
     retval |= test_routing(ag); SINGLE_FAILURE_CHECK();
     retval |= test_dialog(ag); SINGLE_FAILURE_CHECK();
