@@ -117,7 +117,6 @@ struct outbound {
 
   char const *ob_instance;	/**< Our instance ID */
   int32_t ob_reg_id;		/**< Flow-id */
-  char const *ob_features;	/**< Feature parameters for rcontact */
   sip_contact_t *ob_rcontact;	/**< Our contact */
   sip_contact_t *ob_dcontact;	/**< Contact for dialogs */
   sip_contact_t *ob_previous;	/**< Stale contact */
@@ -294,7 +293,8 @@ int outbound_set_options(outbound_t *ob,
     else if (MATCH(use-upnp) || MATCH(use_upnp)) prefs->use_upnp = value;
     else if (MATCH(use-stun) || MATCH(use_stun)) prefs->use_stun = value;
     else
-      SU_DEBUG_1(("outbound_t: unknown option \"%.*s\"\n", (int)len, s));
+      SU_DEBUG_1(("outbound(%p): unknown option \"%.*s\"\n",
+		  (void *)ob->ob_owner, (int)len, s));
 
     s += len;
     len = strspn(s, " \t\n\r,;");
@@ -304,7 +304,8 @@ int outbound_set_options(outbound_t *ob,
   }
 
   if (s && s[0]) {
-    SU_DEBUG_1(("outbound_t: invalid options \"%s\"\n", options));
+    SU_DEBUG_1(("outbound(%p): invalid options \"%s\"\n", 
+		(void *)ob->ob_owner, options));
     return -1;
   }
 
@@ -315,7 +316,8 @@ int outbound_set_options(outbound_t *ob,
 	prefs->use_socks ||
 	prefs->use_upnp ||
 	prefs->use_stun)) {
-    SU_DEBUG_1(("outbound(%p): no nat traversal method given\n", ob->ob_owner));
+    SU_DEBUG_1(("outbound(%p): no nat traversal method given\n",
+		(void *)ob->ob_owner));
   }
 
   ob->ob_prefs = *prefs;
@@ -324,60 +326,27 @@ int outbound_set_options(outbound_t *ob,
   return 0;
 }
 
-/** Set the feature string (added to the Contact header when registering). */
-int outbound_set_features(outbound_t *ob, char *features)
+/* ---------------------------------------------------------------------- */
+
+/** Obtain contacts for REGISTER */
+int outbound_get_contacts(outbound_t *ob, 
+			  sip_contact_t **return_current_contact, 
+			  sip_contact_t **return_previous_contact)
 {
-  char *old_features = (char *)ob->ob_features;
-  char *new_features = su_strdup(ob->ob_home, features);
-
-  if (features && !new_features)
-    return -1;
-
-  ob->ob_features = new_features;
-  su_free(ob->ob_home, old_features);
+  if (ob) {
+    if (ob->ob_contacts)
+      *return_current_contact = ob->ob_rcontact;
+    *return_previous_contact = ob->ob_previous;
+  }
   return 0;
 }
 
-/* ---------------------------------------------------------------------- */
-
-/** Hook for sending register request with extra outbound-ish headers. */
-nta_outgoing_t *outbound_register_request(outbound_t *ob, int terminating, 
-					  sip_contact_t *stack_contact,
-					  nta_agent_t *nta,
-					  nta_response_f *callback,
-					  nta_outgoing_magic_t *magic,
-					  url_string_t *next_hop,
-					  msg_t *msg,
-					  tag_type_t tag, tag_value_t value,
-					  ...)
+/** REGISTER request has been sent */
+int outbound_start_registering(outbound_t *ob)
 {
-  sip_contact_t *previous_contact = NULL;
-  ta_list ta;
-  nta_outgoing_t *orq;
-
-  if (stack_contact) {
-    if (ob) {
-      if (ob->ob_contacts)
-	stack_contact = ob->ob_rcontact;
-      previous_contact = ob->ob_previous;
-    }
-  }
-
-  ta_start(ta, tag, value);
-
-  orq = nta_outgoing_mcreate(nta, callback, magic, next_hop, msg,
-			     TAG_IF(previous_contact,
-				    SIPTAG_CONTACT(previous_contact)),
-			     TAG_IF(stack_contact,
-				    SIPTAG_CONTACT(stack_contact)),
-			     ta_tags(ta));
-
-  ta_end(ta);
-
-  if (orq && ob)
+  if (ob)
     ob->ob_registering = 1;
-
-  return orq;
+  return 0;
 }
 
 /** Process response to REGISTER request */
@@ -391,17 +360,16 @@ int outbound_register_response(outbound_t *ob,
   if (!ob)
     return 0;
 
-  assert(!request || request->sip_request);
-  assert(!response || response->sip_status);
-  
-  if (!response || !request)
-    return 0;
-
   if (terminating) {
-    ob->ob_registered = ob->ob_registering = 0;
+    ob->ob_registering = ob->ob_registered = 0;
     return 0;			/* Cleanup is done separately */
   }  
 
+  if (!response || !request)
+    return 0;
+
+  assert(request->sip_request); assert(response->sip_status);
+  
   reregister = outbound_check_for_nat(ob, request, response);
   if (reregister)
     return reregister;
@@ -423,7 +391,13 @@ int outbound_register_response(outbound_t *ob,
 }
 
 
-/** @internal Check if there is a NAT between us and registrar */
+/** @internal Check if there is a NAT between us and registrar.
+ *
+ * @retval -1 upon an error
+ * @retval #ob_register_ok (0) if the registration was OK
+ * @retval #ob_reregister (1) if client needs to re-register
+ * @retval #ob_reregister_now (2) if client needs to re-register immediately
+ */
 static
 int outbound_check_for_nat(outbound_t *ob,
 			   sip_t const *request,
@@ -457,18 +431,18 @@ int outbound_check_for_nat(outbound_t *ob,
   if (!m || binding_changed >= ob_nat_changed) {
     if (ob->ob_stun) {
       /* Use STUN? */
-      return 1;
+      return ob_reregister;
     }
     else if (ob->ob_upnp) {
       /* Use UPnP */
-      return 1;
+      return ob_reregister;
     }
     else {
       if (outbound_contacts_from_via(ob, response->sip_via) < 0)
         return -1;
     }
 
-    return 2;
+    return ob_reregister_now;
   }
 
   return 0;
@@ -512,7 +486,7 @@ int outbound_nat_detect(outbound_t *ob,
   if (!host_is_ip_address(received)) {
     if (received[0])
       SU_DEBUG_3(("outbound(%p): Via with invalid received=%s\n",
-		  ob->ob_owner, received));
+		  (void *)ob->ob_owner, received));
     return 0;
   }
 
@@ -530,14 +504,14 @@ int outbound_nat_detect(outbound_t *ob,
 
   if (!nat_detected) {
     SU_DEBUG_1(("outbound(%p): detected NAT: %s != %s\n",
-		ob->ob_owner, v->v_host, received));
+		(void *)ob->ob_owner, v->v_host, received));
     if (ob->ob_oo && ob->ob_oo->oo_status)
       ob->ob_oo->oo_status(ob->ob_owner, ob, 101, "NAT detected", TAG_END());
   }
   else {
     SU_DEBUG_1(("outbound(%p): NAT binding changed: "
 		"[%s]:%s != [%s]:%s\n",
-		ob->ob_owner, nat_detected, nat_port, received, rport));
+		(void *)ob->ob_owner, nat_detected, nat_port, received, rport));
     if (ob->ob_oo && ob->ob_oo->oo_status)
       ob->ob_oo->oo_status(ob->ob_owner, ob, 102, "NAT binding changed", TAG_END());
   }
@@ -709,21 +683,32 @@ static int create_keepalive_message(outbound_t *ob, sip_t const *regsip)
 {
   msg_t *msg = nta_msg_create(ob->ob_nta, MSG_FLG_COMPACT), *previous;
   sip_t *osip = sip_object(msg);
-  sip_accept_contact_t *ac;
-
-  char const *p1 = ob->ob_instance;
-  char const *p2 = ob->ob_features;
+  sip_contact_t *m = ob->ob_rcontact;
 
   unsigned d = ob->ob_keepalive.interval;
 
   assert(regsip); assert(regsip->sip_request);
 
-  if (p1 || p2) {
-    ac = sip_accept_contact_format(msg_home(msg), "*;require;explicit;%s%s%s",
-				   p1 ? p1 : "",
-				   p2 && p2 ? ";" : "",
-				   p2 ? p2 : "");
-    msg_header_insert(msg, NULL, (void *)ac);
+  if (m && m->m_params) {
+    sip_accept_contact_t *ac;
+    size_t i;
+    int features = 0;
+
+    ac = sip_accept_contact_make(msg_home(msg), "*;require;explicit");
+
+    for (i = 0; m->m_params[i]; i++) {
+      char const *s = m->m_params[i];
+      if (!sip_is_callerpref(s))
+	continue;
+      features++;
+      s = su_strdup(msg_home(msg), s);
+      msg_header_add_param(msg_home(msg), ac->cp_common, s);
+    }
+    
+    if (features)
+      msg_header_insert(msg, NULL, (void *)ac);
+    else
+      msg_header_free(msg_home(msg), (void *)ac);
   }
 
   if (0 >
@@ -877,18 +862,19 @@ static int response_to_keepalive_options(outbound_t *ob,
     if (loglevel >= SU_LOG->log_level) {
       su_llog(SU_LOG, loglevel,       
 	      "outbound(%p): %s <" URL_PRINT_FORMAT ">\n",
-	      ob->ob_owner, failed ? "FAILED to validate" : "validated", 
+	      (void *)ob->ob_owner,
+	      failed ? "FAILED to validate" : "validated",
 	      URL_PRINT_ARGS(ob->ob_rcontact->m_url));
       if (failed)
 	su_llog(SU_LOG, loglevel, "outbound(%p): FAILED with %u %s\n", 
-		ob->ob_owner, status, phrase);
+		(void *)ob->ob_owner, status, phrase);
     }
 
     if (failed) 
       ob->ob_oo->oo_probe_error(ob->ob_owner, ob, status, phrase, TAG_END());
   }
   else if (status == 408) {
-    SU_DEBUG_1(("outbound(%p): keepalive timeout\n", ob->ob_owner));
+    SU_DEBUG_1(("outbound(%p): keepalive timeout\n", (void *)ob->ob_owner));
     ob->ob_oo->oo_keepalive_error(ob->ob_owner, ob, status, phrase, TAG_END());
     return 0;
   }
@@ -982,7 +968,7 @@ int outbound_process_request(outbound_t *ob,
 
   if (ob->ob_keepalive.validating) {
     SU_DEBUG_1(("outbound(%p): registration check OPTIONS received\n", 
-		ob->ob_owner));
+		(void *)ob->ob_owner));
     ob->ob_keepalive.validated = 1;
   }
 
@@ -1018,48 +1004,16 @@ int outbound_contacts_from_via(outbound_t *ob, sip_via_t const *via)
 
   v = v0; *v0 = *via; v0->v_next = NULL;
 
-  dcontact = ob->ob_oo->oo_contact(ob->ob_owner, home,
+  dcontact = ob->ob_oo->oo_contact(ob->ob_owner, home, 1,
 				   NULL, v, v->v_protocol, NULL);
 
   if (ob->ob_instance && ob->ob_reg_id != 0)
     snprintf(reg_id_param, sizeof reg_id_param, ";reg-id=%u", ob->ob_reg_id);
 
-  rcontact = ob->ob_oo->oo_contact(ob->ob_owner, home,
+  rcontact = ob->ob_oo->oo_contact(ob->ob_owner, home, 0,
 				   NULL, v, v->v_protocol, 
-				   ob->ob_features ? ob->ob_features : "",
 				   ob->ob_instance, reg_id_param, NULL);
     
-#if 0
-  char *uri;
-
-  /* uri contains < > */
-  uri = sip_contact_string_from_via(NULL, via, NULL, v->v_protocol);
-
-  dcontact = sip_contact_make(home, uri);
-
-
-
-  if (ob->ob_instance) {
-    char reg_id[20];
-
-    if (ob->ob_instance && ob->ob_reg_id)
-      snprintf(reg_id, sizeof reg_id, ";reg-id=%u", ob->ob_reg_id);
-    else
-      strcpy(reg_id, "");
-
-    rcontact = sip_contact_format(home, "%s;%s%s%s%s",
-				  uri, ob->ob_instance, reg_id,
-				  ob->ob_features ? ";" : "",
-				  ob->ob_features ? ob->ob_features : "");
-  }
-  else if (ob->ob_features)
-    rcontact = sip_contact_format(home, "%s;%s", uri, ob->ob_features);
-  else
-    rcontact = dcontact;
-
-  free(uri);
-#endif
-
   v = sip_via_dup(home, v);
 
   if (!rcontact || !dcontact || !v) {
@@ -1137,6 +1091,9 @@ int outbound_set_contact(outbound_t *ob,
   m3 = ob->ob_previous;
 
   if (terminating) {
+    if (ob->ob_by_stack && application_contact == NULL)
+      return 0;
+    
     if (ob->ob_contacts)
       previous = ob->ob_rcontact;
   }
@@ -1147,11 +1104,14 @@ int outbound_set_contact(outbound_t *ob,
       previous = ob->ob_contacts ? ob->ob_rcontact : NULL;
     }
   }
+  else if (ob->ob_by_stack) {
+    return 0;    /* Xyzzy - nothing happens */
+  }
   else if (v) {
     char const *tport = !v->v_next ? v->v_protocol : NULL; 
     char reg_id_param[20];
 
-    dcontact = ob->ob_oo->oo_contact(ob->ob_owner, home,
+    dcontact = ob->ob_oo->oo_contact(ob->ob_owner, home, 1, 
 				     NULL, v, tport, NULL);
     if (!dcontact)
       return -1;
@@ -1159,9 +1119,8 @@ int outbound_set_contact(outbound_t *ob,
     if (ob->ob_instance && ob->ob_reg_id != 0)
       snprintf(reg_id_param, sizeof reg_id_param, ";reg-id=%u", ob->ob_reg_id);
 
-    rcontact = ob->ob_oo->oo_contact(ob->ob_owner, home,
+    rcontact = ob->ob_oo->oo_contact(ob->ob_owner, home, 0,
 				     NULL, v, v->v_protocol, 
-				     ob->ob_features ? ob->ob_features : "",
 				     ob->ob_instance, reg_id_param, NULL);
     if (!rcontact)
       return -1;
@@ -1172,9 +1131,11 @@ int outbound_set_contact(outbound_t *ob,
       previous = ob->ob_contacts ? ob->ob_rcontact : NULL;
     }
   }
-
+    
   ob->ob_by_stack = application_contact == NULL;
 
+  ob->ob_contacts = rcontact != NULL;
+
   ob->ob_rcontact = rcontact;
   ob->ob_dcontact = dcontact;
   ob->ob_previous = previous;
@@ -1205,6 +1166,11 @@ sip_contact_t const *outbound_dialog_contact(outbound_t const *ob)
     return ob->ob_dcontact;
 }
 
+sip_contact_t const *outbound_dialog_gruu(outbound_t const *ob)
+{
+  return ob ? ob->ob_gruu : NULL;
+}
+
 /* ---------------------------------------------------------------------- */
 
 
