@@ -52,6 +52,7 @@ struct binding;
 #include <sofia-sip/auth_module.h>
 #include <sofia-sip/su_tagarg.h>
 #include <sofia-sip/msg_addr.h>
+#include <sofia-sip/hostdomain.h>
 
 #include <stdlib.h>
 #include <assert.h>
@@ -228,9 +229,10 @@ test_proxy_init(su_root_t *root, struct proxy *proxy)
 				  URL_STRING_MAKE("sip:0.0.0.0:*"),
 				  NULL, NULL,
 				  NTATAG_UA(0),
+				  NTATAG_CANCEL_487(0),
 				  NTATAG_SERVER_RPORT(1),
 				  NTATAG_CLIENT_RPORT(1),
-				  TAG_END());
+				  TAG_NEXT(proxy->tags));
 
   proxy->transport_contacts = create_transport_contacts(proxy);
 
@@ -308,7 +310,7 @@ test_proxy_deinit(su_root_t *root, struct proxy *proxy)
   free(proxy->tags);
 }
 
-/* Create tst proxy object */
+/* Create test proxy object */
 struct proxy *test_proxy_create(su_root_t *root,
 				tag_type_t tag, tag_value_t value, ...)
 {
@@ -373,6 +375,27 @@ void test_proxy_get_expiration(struct proxy *p,
   }
 }
 
+void test_proxy_set_session_timer(struct proxy *p,
+				  sip_time_t session_expires, 
+				  sip_time_t min_se)
+{
+  if (p) {
+    p->prefs.session_expires = session_expires;
+    p->prefs.min_se = min_se;
+  }
+}
+
+void test_proxy_get_session_timer(struct proxy *p,
+				  sip_time_t *return_session_expires,
+				  sip_time_t *return_min_se)
+{
+  if (p) {
+    if (return_session_expires)
+      *return_session_expires = p->prefs.session_expires;
+    if (return_min_se) *return_min_se = p->prefs.min_se;
+  }
+}
+
 /* ---------------------------------------------------------------------- */
 
 static sip_contact_t *create_transport_contacts(struct proxy *p)
@@ -425,6 +448,7 @@ int proxy_request(struct proxy *proxy,
 
   sip_session_expires_t *x = NULL, x0[1];
   sip_min_se_t *min_se = NULL, min_se0[1];
+  char const *require = NULL;
 
   mf = sip->sip_max_forwards;
 
@@ -445,24 +469,33 @@ int proxy_request(struct proxy *proxy,
   }
 
   if (method == sip_method_invite) {
-    if (!sip->sip_min_se || 
-	sip->sip_min_se->min_delta < proxy->prefs.min_se) {
-      min_se = sip_min_se_init(min_se0);
-      min_se->min_delta = proxy->prefs.min_se;
+    if (proxy->prefs.min_se) {
+      if (!sip->sip_min_se || 
+	  sip->sip_min_se->min_delta < proxy->prefs.min_se) {
+	min_se = sip_min_se_init(min_se0);
+	min_se->min_delta = proxy->prefs.min_se;
+      }
+
+      if (sip->sip_session_expires
+	  && sip->sip_session_expires->x_delta < proxy->prefs.min_se
+	  && sip_has_supported(sip->sip_supported, "timer")) {
+	if (min_se == NULL)
+	  min_se = sip->sip_min_se; assert(min_se);
+	nta_incoming_treply(irq, SIP_422_SESSION_TIMER_TOO_SMALL,
+			    SIPTAG_MIN_SE(min_se),
+			    TAG_END());
+	return 422;
+      }
     }
 
-    if (!sip->sip_session_expires) {
-      x = sip_session_expires_init(x0);
-      x->x_delta = proxy->prefs.session_expires;
-    }
-    else if (sip->sip_session_expires->x_delta < proxy->prefs.min_se
-	     && sip_has_supported(sip->sip_supported, "timer")) {
-      if (min_se == NULL)
-	min_se = sip->sip_min_se; assert(min_se);
-      nta_incoming_treply(irq, SIP_422_SESSION_TIMER_TOO_SMALL,
-			  SIPTAG_MIN_SE(min_se),
-			  TAG_END());
-      return 422;
+    if (proxy->prefs.session_expires) {
+      if (!sip->sip_session_expires ||
+	  sip->sip_session_expires->x_delta > proxy->prefs.session_expires) {
+	x = sip_session_expires_init(x0);
+	x->x_delta = proxy->prefs.session_expires;
+	if (!sip_has_supported(sip->sip_supported, "timer"))
+	  require = "timer";
+      }
     }
   }
 
@@ -526,6 +559,7 @@ int proxy_request(struct proxy *proxy,
 				   SIPTAG_REQUEST(rq),
 				   SIPTAG_SESSION_EXPIRES(x),
 				   SIPTAG_MIN_SE(min_se),
+				   SIPTAG_REQUIRE_STR(require),
 				   TAG_END());
   if (t->client == NULL) {
     proxy_transaction_destroy(t);
@@ -707,8 +741,8 @@ static int binding_update(struct proxy *p,
 sip_contact_t *binding_contacts(su_home_t *home, struct binding *bindings);
 
 int process_register(struct proxy *proxy,
-       	      nta_incoming_t *irq,
-       	      sip_t const *sip)
+		     nta_incoming_t *irq,
+		     sip_t const *sip)
 {
   auth_status_t *as;
   msg_t *msg;
@@ -751,6 +785,13 @@ static int process_register2(struct proxy *p,
 			     sip_t const *sip)
 {
   struct registration_entry *e = NULL;
+  sip_contact_t *m = sip->sip_contact;
+  sip_via_t *v = sip->sip_via;
+
+  if (m && v && v->v_received && m->m_url->url_host
+      && strcasecmp(v->v_received, m->m_url->url_host) 
+      && host_is_ip_address(m->m_url->url_host))
+    return set_status(as, 406, "Unacceptable Contact");
 
   auth_mod_check_client(p->auth, as, sip->sip_authorization,
 			registrar_challenger);
