@@ -1061,7 +1061,7 @@ tagi_t *sdptag_session_dup(tagi_t *dst, tagi_t const *src, void **bb)
   b += STRUCT_ALIGN(b);
   srcsdp = (sdp_session_t *)src->t_value;
 
-  sdp = session_dup(&b, srcsdp);
+  sdp = srcsdp ? session_dup(&b, srcsdp) : NULL;
 
   dst->t_tag = src->t_tag;
   dst->t_value = (tag_value_t)sdp;
@@ -1115,7 +1115,7 @@ tag_class_t sdptag_session_class[1] =
 /* ---------------------------------------------------------------------- */
 
 /* Compare two string pointers */
-static inline 
+su_inline 
 int str0cmp(char const *a, char const *b)
 {
   if (a == NULL) a = "";
@@ -1124,7 +1124,7 @@ int str0cmp(char const *a, char const *b)
 }
 
 /* Compare two string pointers ignoring case. */
-static inline 
+su_inline 
 int str0casecmp(char const *a, char const *b)
 {
   if (a == NULL) a = "";
