@@ -229,7 +229,8 @@ typedef enum
   sdp_media_data,			/**< Bulk data transfer */
   sdp_media_control,			/**< Additional conference control */
   sdp_media_message,			/**< Messaging sessions*/
-  sdp_media_image			/**< Image browsing sessions for JPIP */
+  sdp_media_image,			/**< Image browsing sessions for JPIP, and T.38 */
+  sdp_media_red				/**< Redundancy. @NEW_1_12_4 */
 } sdp_media_e;
 
 /** Media transport protocol. */
@@ -240,6 +241,7 @@ typedef enum
   sdp_proto_udp = 17,			/**< Plain UDP */
   sdp_proto_rtp = 256,			/**< RTP/AVP */
   sdp_proto_srtp = 257,			/**< RTP/SAVP  */
+  sdp_proto_udptl = 258,		/**< UDPTL. @NEW_1_12_4  */
   sdp_proto_tls = 511,			/**< TLS over TCP */
   sdp_proto_any = 512		        /**< * wildcard */
 } sdp_proto_e;
@@ -283,7 +285,7 @@ struct sdp_media_s
   sdp_attribute_t  *m_attributes;	/**< Media attributes */
 
   void             *m_user;	        /**< User data. */
-
+  
   /** Rejected media */
   unsigned          m_rejected : 1;     
   /** Inactive, recvonly, sendonly, sendrecv */
@@ -528,7 +530,8 @@ typedef struct sdp_parser_s sdp_parser_t;
 typedef sdp_parser_t  *sdp_parser;
 
 SOFIAPUBFUN sdp_parser_t *sdp_parse(su_home_t *,
-				    char const msg[], int msgsize, int flags);
+				    char const msg[], issize_t msgsize,
+				    int flags);
 SOFIAPUBFUN char const *sdp_parsing_error(sdp_parser_t *p);
 SOFIAPUBFUN sdp_session_t *sdp_session(sdp_parser_t *p);
 SOFIAPUBFUN void sdp_parser_free(sdp_parser_t *p);
@@ -544,10 +547,10 @@ typedef struct sdp_printer_s sdp_printer_t;
 typedef sdp_printer_t *sdp_printer;
 
 SOFIAPUBFUN sdp_printer_t *sdp_print(su_home_t *, sdp_session_t const *session, 
-				     char msgbuf[], int maxmsgsize, int flags);
+				     char msgbuf[], isize_t maxmsgsize, int flags);
 SOFIAPUBFUN char const *sdp_printing_error(sdp_printer_t *p);
 SOFIAPUBFUN char const *sdp_message(sdp_printer_t *p);
-SOFIAPUBFUN int   sdp_message_size(sdp_printer_t *p);
+SOFIAPUBFUN isize_t sdp_message_size(sdp_printer_t *p);
 SOFIAPUBFUN void sdp_printer_free(sdp_printer_t *p);
 
 #define sdp_mapped_attribute_find sdp_attribute_mapped_find
