@@ -218,8 +218,8 @@ issize_t sip_proxy_authenticate_e(char b[], isize_t bsiz, sip_header_t const *h,
  *
  * The Proxy-Authorization header consists of credentials containing the
  * authentication information of the user agent for the proxy and/or realm
- * of the resource being requested.  Its syntax is defined in [H14.34,
- * S10.32] (as follows:
+ * of the resource being requested.  Its syntax is defined in @RFC3261
+ * as follows:
  *
  * @code
  *    Proxy-Authorization  = "Proxy-Authorization" ":" credentials
@@ -273,16 +273,17 @@ issize_t sip_proxy_authorization_e(char b[], isize_t bsiz, sip_header_t const *h
  *
  * The WWW-Authenticate header consists of at least one challenge that
  * indicates the authentication scheme(s) and parameters applicable to the
- * Request-URI.  Its syntax is defined in [H14.47, S10.48] as
+ * Request-URI.  Its syntax is defined in @RFC3261 as
  * follows:
  *
  * @code
- *    WWW-Authenticate  = "WWW-Authenticate" ":" 1#challenge
- *    challenge         = auth-scheme 1*SP #auth-param
+ *    WWW-Authenticate  = "WWW-Authenticate" HCOLON challenge
+ *    challenge         =  ("Digest" LWS digest-cln *(COMMA digest-cln))
+ *                      / other-challenge
+ *    other-challenge   =  auth-scheme LWS auth-param *(COMMA auth-param)
  * @endcode
  *
- * @todo Currently the parser fails if several challenges are included one
- * header.
+ * See @ProxyAuthenticate for the definition of \<digest-cln\>.
  *
  * The parsed WWW-Authenticate header
  * is stored in #sip_www_authenticate_t structure.
@@ -350,11 +351,11 @@ issize_t sip_www_authenticate_e(char b[], isize_t bsiz, sip_header_t const *h, i
  *
  * The #sip_authentication_info_t is defined as follows:
  * @code
- * typedef struct msg_list_s
+ * typedef struct msg_auth_info_s
  * {
- *   msg_common_t       k_common[1];  // Common fragment info
- *   msg_list_t        *k_next;	      // Dummy link to next header
- *   msg_param_t       *k_items;      // List of ainfo
+ *   msg_common_t       ai_common[1];  // Common fragment info
+ *   msg_error_t       *ai_next;       // Dummy link to next header
+ *   msg_param_t       *ai_items;      // List of ainfo
  * } sip_authentication_info_t;
  * @endcode
  */
@@ -408,18 +409,18 @@ issize_t sip_authentication_info_e(char b[], isize_t bsiz, sip_header_t const *h
  */
 
 /**@ingroup sip_proxy_authentication_info
- * @typedef typedef struct sip_proxy_authentication_info_s sip_proxy_authentication_info_t;
+ * @typedef typedef struct msg_authentication_info_s sip_proxy_authentication_info_t;
  *
  * The structure #sip_proxy_authentication_info_t contains representation of SIP
  * @ProxyAuthenticationInfo header.
  *
  * The #sip_proxy_authentication_info_t is defined as follows:
  * @code
- * typedef struct msg_list_s
+ * typedef struct msg_auth_info_s
  * {
- *   msg_common_t       k_common[1];  // Common fragment info
- *   msg_list_t        *k_next;	      // Dummy link to next header
- *   msg_param_t       *k_items;      // List of ainfo
+ *   msg_common_t       ai_common[1];  // Common fragment info
+ *   msg_error_t       *ai_next;       // Dummy link to next header
+ *   msg_param_t       *ai_items;      // List of ainfo
  * } sip_proxy_authentication_info_t;
  * @endcode
  *
@@ -442,7 +443,7 @@ issize_t sip_proxy_authentication_info_d(su_home_t *home, sip_header_t *h,
 issize_t sip_proxy_authentication_info_e(char b[], isize_t bsiz, 
 					 sip_header_t const *h, int f)
 {
-  assert(sip_is_authentication_info(h));
+  assert(sip_is_proxy_authentication_info(h)); /* This is soo popular */
   return msg_list_e(b, bsiz, h, f);
 }
 
