@@ -48,22 +48,9 @@
 
 struct su_root_s;
 
-typedef struct su_cloned_s {
-  struct su_root_s *sc_root;
-  int *sc_wait;
-#if SU_HAVE_PTHREADS
-  pthread_t  sc_tid;
-  pthread_mutex_t sc_pause[1];
-  pthread_cond_t sc_resume[1];
-  int sc_paused;
-#endif  
-} su_cloned_t;
-
 #define SU_ROOT_MAGIC_T struct su_root_magic_s
 #define SU_WAKEUP_ARG_T struct su_wakeup_arg_s
 #define SU_TIMER_ARG_T  struct su_timer_arg_s
-#define SU_CLONE_T      su_msg_t
-#define SU_MSG_ARG_T    struct su_cloned_s
 
 #include "su_port.h"
 #include "sofia-sip/su_alloc.h"
@@ -136,11 +123,11 @@ su_task_r const su_task_null = SU_TASK_R_INIT;
 
 #define SU_TASK_ZAP(t, f) \
   while (t->sut_port) { \
-   SU_PORT_DECREF(t->sut_port, f); t->sut_port = NULL; break; }
+   su_port_decref(t->sut_port, #f); t->sut_port = NULL; break; }
 
 #define SU_TASK_ZAPP(t, f) \
   do { if (t->sut_port) { \
-   SU_PORT_DECREF(t->sut_port, f); t->sut_port = NULL; } \
+   su_port_decref(t->sut_port, #f); t->sut_port = NULL; } \
    t->sut_root = NULL; } while(0)
 
 /**
@@ -186,7 +173,7 @@ _su_task_r su_task_new(su_task_r task, su_root_t *root, su_port_t *port)
 
   task->sut_root = root;
   if ((task->sut_port = port)) {
-    SU_PORT_INCREF(port, su_task_new);
+    su_port_incref(port, "su_task_new");
   }
   return task;
 }
@@ -207,14 +194,14 @@ void su_task_copy(su_task_r dst, su_task_r const src)
 
   port = src->sut_port;
   if (port) {
-    SU_PORT_INCREF(port, su_task_copy);
+    su_port_incref(port, "su_task_copy");
   }
 
   dst[0] = src[0];
 }
 
 #define SU_TASK_COPY(d, s, by) (void)((d)[0]=(s)[0], \
-  (s)->sut_port?(void)SU_PORT_INCREF(s->sut_port, by):(void)0)
+  (s)->sut_port?(void)su_port_incref(s->sut_port, #by):(void)0)
 
 /**
  * Moves a task handle.
@@ -236,15 +223,25 @@ void su_task_move(su_task_r dst, su_task_r src)
  * @param a  First task
  * @param b  Second task
  * 
- * @return 
- *   The function @c su_task_cmp() returns negative number, if a < b,
- *   positive number, if a > b, and 0, if a == b.
+ * @retval negative number, if a < b
+ * @retval positive number, if a > b
+ * @retval 0, if a == b.
  */
-int  su_task_cmp(su_task_r const a, su_task_r const b)
+int su_task_cmp(su_task_r const a, su_task_r const b)
 {
-  int retval = a->sut_port - b->sut_port;
+  intptr_t retval = (char *)a->sut_port - (char *)b->sut_port;
 
-  return retval ? retval : (char *)a->sut_root - (char *)b->sut_root;
+  if (retval == 0)
+    retval = (char *)a->sut_root - (char *)b->sut_root;
+
+  if (sizeof(retval) != sizeof(int)) {
+    if (retval < 0)
+      retval = -1;
+    else if (retval > 0)
+      retval = 1;
+  }
+
+  return (int)retval;
 }
 
 /**
@@ -288,8 +285,8 @@ int su_task_attach(su_task_r self, su_root_t *root)
  * @param self task handle
  *
  * @return 
- * The function @c su_task_root() returns a pointer to root object attached
- * to the task handle, or NULL if no root object has been attached.
+ * A pointer to root object attached to the task handle, or NULL if no root
+ * object has been attached.
  */
 su_root_t *su_task_root(su_task_r const self)
 {
@@ -313,39 +310,15 @@ int su_task_detach(su_task_r self)
  * 
  * @param task task handle
  *
- * @return The function @c su_task_timers() returns a timer list of the
- * task. If there are no timers, it returns NULL.
+ * @return A timer list of the task. If there are no timers, it returns
+ * NULL.
  */
 su_timer_t **su_task_timers(su_task_r const task)
 {
   return task ? su_port_timers(task->sut_port) : NULL;
 }
 
-#if SU_HAVE_PTHREADS
-
-struct su_task_execute
-{
-  pthread_mutex_t mutex[1];
-  pthread_cond_t cond[1];
-  int (*function)(void *);
-  void *arg;
-  int *return_value;
-};
-
-static void _su_task_execute(su_root_magic_t *m,
-			     su_msg_r msg,
-			     su_msg_arg_t *a)
-{
-  struct su_task_execute *frame = (void *)a;
-  pthread_mutex_lock(frame->mutex);
-  *frame->return_value = frame->function(frame->arg);
-  pthread_cond_signal(frame->cond);
-  pthread_mutex_unlock(frame->mutex);
-}
-
-#endif
-
-/** Execute by task thread
+/** Execute the @a function by @a task thread.
  *
  * @retval 0 if successful
  * @retval -1 upon an error
@@ -354,42 +327,20 @@ int su_task_execute(su_task_r const task,
 		    int (*function)(void *), void *arg,
 		    int *return_value)
 {
-  int value;
+  if (function == NULL)
+    return (errno = EFAULT), -1;
 
   if (!su_port_own_thread(task->sut_port)) {
-#if SU_HAVE_PTHREADS
-    su_msg_r m = SU_MSG_R_INIT;
-    struct su_task_execute *frame;
-
-    if (su_msg_create(m, task, su_task_null,
-		      _su_task_execute, (sizeof *frame)) < 0)
-      return -1;
-
-    frame = (void *)su_msg_data(m);
-    pthread_mutex_init(frame->mutex, NULL);
-    pthread_cond_init(frame->cond, NULL);
-    frame->function = function;
-    frame->arg = arg;
-    frame->return_value = &value;
-
-    pthread_mutex_lock(frame->mutex);
-
-    if (su_msg_send(m) < 0) {
-      su_msg_destroy(m);
-      return -1;
-    }
-
-    pthread_cond_wait(frame->cond, frame->mutex);
-#else
-    return -1;
-#endif
+    return su_port_execute(task, function, arg, return_value);
   }
-  else
-    value = function(arg);
-  if (return_value)
-    *return_value = value;
+  else {
+    int value = function(arg);
 
-  return 0;
+    if (return_value)
+      *return_value = value;
+
+    return 0;
+  }
 }
 
 _su_task_r su_task_new(su_task_r task, su_root_t *root, su_port_t *port);
@@ -398,51 +349,6 @@ int su_task_detach(su_task_r self);
 
 int su_timer_reset_all(su_timer_t **t0, su_task_r);
 
-/**@ingroup su_wait
- * 
- * @page su_clone_t Clone Objects
- *
- * The process may be divided into many tasks via cloning. Several tasks may
- * run in context of one thread, or each task may be run by its own thread. 
- * However, only a single thread can execute code within a task. There can
- * be a 1-to-N mapping from thread to tasks. Thus, software using tasks can
- * be executed by multiple threads in a multithreaded environment and by a
- * single thread in a singlethreaded environment.
- * 
- * The clones are useful for handling tasks that can be executed by a
- * separate threads, but which do not block excessively. When threads are
- * not available or they are not needed, clones can also be run in a
- * single-threaded mode. Running in single-threaded mode is especially
- * useful while debugging.
- * 
- * A clone task is created with function su_clone_start(). Each clone has
- * its own root object (su_root_t), which holds a context pointer
- * (su_root_magic_t *). The context object can be different from that of 
- * parent task.
- *
- * When a clone is started, the clone initialization function is called. The
- * initialization function should do whatever initialization there is to be
- * performed, register I/O events and timers, and then return. If the
- * initialization is successful, the clone task reverts to run the event
- * loop and invoking the event callbacks until its parent stops it by
- * calling su_clone_wait() which invokes the deinit function. The clone task
- * is destroyed when the deinit function returns. 
- *
- * The public API consists of following functions:
- *    - su_clone_start()
- *    - su_clone_task()
- *    - su_clone_wait()
- *    - su_clone_forget()
- *
- * @note 
- * There is only one event loop for each thread which can be shared by
- * multiple clone tasks. Therefore, the clone tasks can not explicitly run
- * or step the event loop, but they are limited to event callbacks. A clone
- * task may not call su_root_break(), su_root_run() or su_root_step().
- */
-
-static void su_root_deinit(su_root_t *self);
-
 /* Note that is *not* necessary same as su_root_t,
  * as su_root_t can be extended */
 
@@ -464,9 +370,13 @@ su_root_t *su_root_create(su_root_magic_t *magic)
   return su_root_create_with_port(magic, su_port_create());
 }
 
-/** Create a reactor object using given message port.
+/**@internal
  *
- * Allocate and initialize the instance of su_root_t.
+ * Create a reactor object using given message port.
+ *
+ * Allocate and initialize the instance of su_root_t. Note that this
+ * function always uses a reference to su_port_t, even when creating the
+ * root fails.
  *
  * @param magic     pointer to user data
  * @param port      pointer to a message port
@@ -481,7 +391,7 @@ su_root_t *su_root_create_with_port(su_root_magic_t *magic,
   if (!port)
     return NULL;
 
-  self = su_salloc(NULL, sizeof(struct su_root_s));
+  self = su_salloc(su_port_home(port), sizeof(struct su_root_s));
 
   if (self) {
     self->sur_magic = magic;
@@ -490,15 +400,17 @@ su_root_t *su_root_create_with_port(su_root_magic_t *magic,
 #else
     self->sur_threading = 0;
 #endif
+    /* This one creates a new reference to port */
     su_task_new(self->sur_task, self, port);
-  } else {
-    su_port_decref(port, "su_root_create");
-  }
+    /* ... so we zap the old one below */
+  } 
+
+  su_port_decref(port, "su_root_create_with_port");
 
   return self;
 }
 
-/** Destroy a synchronization object.
+/** Destroy a root object.
  * 
  *  Stop and free an instance of su_root_t
  *
@@ -506,21 +418,14 @@ su_root_t *su_root_create_with_port(su_root_magic_t *magic,
  */
 void su_root_destroy(su_root_t *self)
 {
-  if (self) {
-    assert(SU_ROOT_OWN_THREAD(self));
-    su_root_deinit(self);
-    su_free(NULL, self);
-  }
-}
+  su_port_t *port;
+  int unregistered, reset;
+
+  if (!self)
+    return;
+
+  assert(SU_ROOT_OWN_THREAD(self));
 
-/** @internal Deinitialize a synchronization object.
- *
- *  Deinitialize an instance of su_root_t
- *
- * @param self     pointer to a root object.
- */
-static void su_root_deinit(su_root_t *self)
-{
   self->sur_deiniting = 1;
 
   if (self->sur_deinit) {
@@ -530,17 +435,37 @@ static void su_root_deinit(su_root_t *self)
     deinit(self, magic);
   }
 
-  if (self->sur_port) {
-    int n_w = su_port_unregister_all(self->sur_port, self);
-    int n_t = su_timer_reset_all(su_task_timers(self->sur_task), self->sur_task);
+  port = self->sur_port; assert(port);
 
-    if (n_w || n_t)
-      SU_DEBUG_1(("su_root_deinit: "
-		  "%u registered waits, %u timers\n", n_w, n_t));
-  }
+  unregistered = su_port_unregister_all(port, self);
+  reset = su_timer_reset_all(su_task_timers(self->sur_task), self->sur_task);
 
-  SU_TASK_ZAP(self->sur_parent, su_root_deinit);
-  SU_TASK_ZAP(self->sur_task, su_root_deinit);
+  if (unregistered || reset)
+    SU_DEBUG_1(("su_root_destroy: "
+		"%u registered waits, %u timers\n",
+		unregistered, reset));
+
+  SU_TASK_ZAP(self->sur_parent, su_root_destroy);
+
+  su_free(su_port_home(port), self);
+
+  su_port_decref(port, "su_root_destroy");
+}
+
+/** Get instance name.
+ *
+ * @param self      pointer to a root object
+ *
+ * @return Instance name (e.g., "epoll", "devpoll", "select").
+ *
+ * @NEW_1_12_6
+ */
+char const *su_root_name(su_root_t *self)
+{
+  if (self && self->sur_task->sut_port)
+    return su_port_name(self->sur_task->sut_port);
+  else
+    return NULL;
 }
 
 /** Set the context pointer.
@@ -738,7 +663,6 @@ int su_root_multishot(su_root_t *self, int multishot)
   }
 }
 
-
 /** Run event and message loop.
  *
  * The function su_root_run() runs the root main loop. The root loop waits
@@ -830,12 +754,20 @@ su_duration_t su_root_sleep(su_root_t *self, su_duration_t duration)
 int su_root_yield(su_root_t *self)
 {
   if (self && self->sur_task[0].sut_port) {
-    su_port_t *port = self->sur_task[0].sut_port;
+    su_virtual_port_t *port = (su_virtual_port_t *)self->sur_task[0].sut_port;
+    /* Make sure we have su_port_wait_events extension */
+    if (port->sup_vtable->su_vtable_size >= 
+	offsetof(su_port_vtable_t, su_port_wait_events) 
+	&& port->sup_vtable->su_port_wait_events)
+      return port->sup_vtable->
+	su_port_wait_events(self->sur_task[0].sut_port, 0);
+
     /* Make sure we have su_port_yield extension */
     if (port->sup_vtable->su_vtable_size >= 
 	offsetof(su_port_vtable_t, su_port_yield) 
 	&& port->sup_vtable->su_port_yield)
-      return port->sup_vtable->su_port_yield(port);
+    return port->sup_vtable->
+      su_port_yield(self->sur_task[0].sut_port);
   }
   errno = EINVAL;
   return -1;
@@ -897,467 +829,6 @@ int su_root_remove_prepoll(su_root_t *root)
   return su_port_remove_prepoll(root->sur_port, root);
 }
 
-/* ========================================================================
- * su_clone_t
- */
-
-/* - su_clone_forget() */
-
-#if SU_HAVE_PTHREADS
-struct clone_args
-{
-  su_root_t      * self;
-  su_root_init_f   init;
-  su_root_deinit_f deinit;
-  pthread_mutex_t  mutex;
-  pthread_cond_t   cv;
-  int              retval;
-  su_msg_r         clone;
-  su_root_t const *parent;
-};
-
-static void su_clone_report2(su_root_magic_t *m,
-			     su_msg_r msg,
-			     su_cloned_t *sc);
-
-static void su_clone_signal_parent(void *varg)
-{
-  struct clone_args *arg = (struct clone_args *)varg;
-
-  pthread_mutex_lock(&arg->mutex);
-  pthread_cond_signal(&arg->cv);
-  pthread_mutex_unlock(&arg->mutex);
-}
-
-/** Message function for clone message.
- *
- * This calls the clone task deinitialization function, which should make
- * sure that no more messages are sent by clone task.
- *
- * @sa su_clone_wait()
- */
-static void su_clone_break(su_root_magic_t *m,
-			   su_msg_r msg,
-			   su_cloned_t *sc)
-{
-  su_root_t *root = sc->sc_root;
-
-  root->sur_deiniting = 1;
-
-  if (root->sur_deinit) {
-    su_root_deinit_f deinit = root->sur_deinit;
-    su_root_magic_t *magic = root->sur_magic;
-    root->sur_deinit = NULL;
-    deinit(root, magic);
-  }
-}
-
-/** Delivery report function for clone message.
- *
- * This is executed by parent task. This is the last message sent by clone task.
- */
-static void su_clone_report(su_root_magic_t *m,
-			    su_msg_r msg,
-			    su_cloned_t *sc)
-{
-  su_msg_report(msg, su_clone_report2);
-}
-
-/** Back delivery report function for clone message.
- *
- * This is executed by clone task. It completes the three way handshake and
- * it is used to signal clone that it can destroy its port.
- */
-static void su_clone_report2(su_root_magic_t *m,
-			    su_msg_r msg,
-			    su_cloned_t *sc)
-{
-  su_root_break(sc->sc_root);
-  if (sc->sc_wait)
-    *sc->sc_wait = 0;
-}
-
-static void *su_clone_main(void *varg)
-{
-  struct clone_args *arg = (struct clone_args *)varg;
-  su_root_t *self = arg->self;
-  su_port_t *port;
-  su_cloned_t *sc;
-
-  pthread_cleanup_push(su_clone_signal_parent, varg);
-
-#if SU_HAVE_WINSOCK
-  su_init();
-#endif
-
-  port = su_port_create();
-  if (!port)
-    pthread_exit(NULL);
-  su_port_threadsafe(port);
-  SU_PORT_INCREF(port, su_clone_main);
-
-  /* Change task ownership */
-  SU_PORT_INCREF(self->sur_task->sut_port = port, su_clone_main);
-  self->sur_task->sut_root = self;
-
-  if (su_msg_create(arg->clone,
-		    self->sur_task, su_root_task(arg->parent),
-		    su_clone_break, sizeof(self)) != 0) {
-    su_port_decref(self->sur_port, "su_clone_main");
-    self->sur_port = NULL;
-    pthread_exit(NULL);
-  }
-
-  su_msg_report(arg->clone, su_clone_report);
-
-  sc = su_msg_data(arg->clone);
-  sc->sc_root = self;
-  sc->sc_tid = pthread_self();
-
-  pthread_mutex_init(sc->sc_pause, NULL);
-  pthread_cond_init(sc->sc_resume, NULL);
-  pthread_mutex_lock(sc->sc_pause);
-
-  if (arg->init && arg->init(self, self->sur_magic) != 0) {
-    if (arg->deinit)
-      arg->deinit(self, self->sur_magic);
-    su_msg_destroy(arg->clone);
-    su_port_decref(self->sur_port, "su_clone_main");
-    self->sur_port = NULL;
-    pthread_exit(NULL);
-  }
-
-  arg->retval = 0;
-
-  pthread_cleanup_pop(1);  /* signal change of ownership */
-
-  su_root_run(self);   /* Do the work */
-
-  su_root_destroy(self);   /* Cleanup root */   
-
-  SU_PORT_ZAPREF(port, su_clone_main);
-
-#if SU_HAVE_WINSOCK
-  su_deinit();
-#endif
-
-  return NULL;
-}
-#endif
-
-static void su_clone_xyzzy(su_root_magic_t *m,
-			   su_msg_r msg,
-			   su_cloned_t *sc)
-{
-  su_root_destroy(sc->sc_root);
-  if (sc->sc_wait)
-    *sc->sc_wait = 0;
-}
-
-/** Start a clone task.
- *
- * The function su_clone_start() allocates and initializes a sub-task. 
- * Depending on the settings, a separate thread may be created to execute
- * the sub-task. The sub-task is represented by clone handle to the rest of
- * the application. The function su_clone_start() returns the clone handle
- * in @a return_clone. The clone handle is used to communicate with the
- * newly created clone task using messages.
- *
- * A new #su_root_t object is created for the sub-task with the @a magic as
- * the root context pointer. Because the sub-task may or may not have its
- * own thread, all its activity must be scheduled via this root object. In
- * other words, the sub-task can be schedule
- * -# I/O events with su_root_register()
- * -# timers with su_timer_set(), su_timer_set_at() or su_timer_run()
- * -# messages with su_msg_send().
- *
- * Messages can also be used to pass information between tasks or threads.
- *
- * In multi-threaded implementation, su_clone_start() launches a new thread,
- * and the initialization routine is executed by this newly created thread. 
- * The calling thread blocks until the initialization routine completes. If
- * the initialization routine returns #su_success (0), the sub-task is
- * considered to be created successfully. After the successful
- * initialization, the sub-task continues to execeute the function
- * su_root_run().
- *
- * In single-threaded implementations, just a new root object is created. 
- * The initialization routine is called directly from su_clone_start().
- *
- * If the initalization function @a init fails, the sub-task (either the
- * newly created thread or the current thread executing the su_clone_start()
- * function) calls the deinitialization function, and su_clone_start()
- * returns NULL.
- *
- * @param parent   root to be cloned (may be NULL if multi-threaded)
- * @param return_clone reference to a clone [OUT]
- * @param magic    pointer to user data
- * @param init     initialization function
- * @param deinit   deinitialization function
- *
- * @return 0 if successfull, -1 upon an error.
- *
- * @sa su_root_threading(), su_clone_task(), su_clone_stop(), su_clone_wait(),
- * su_clone_forget().
- */
-int su_clone_start(su_root_t *parent,
-		   su_clone_r return_clone,
-		   su_root_magic_t *magic,
-		   su_root_init_f init,
-		   su_root_deinit_f deinit)
-{
-  su_root_t *child;
-  int retval = -1;
-
-  if (parent) {
-    assert(SU_ROOT_OWN_THREAD(parent));
-    assert(parent->sur_port);
-  }
-#if !SU_HAVE_PTHREADS
-  else {
-    /* if we don't have threads, we *must* have parent root */
-    return -1;
-  }
-#endif
-
-  child = su_salloc(NULL, sizeof(struct su_root_s));
-
-#if SU_HAVE_PTHREADS
-  if (child && (parent == NULL || parent->sur_threading)) {
-    struct clone_args arg = {
-      NULL, NULL, NULL,
-      PTHREAD_MUTEX_INITIALIZER,
-      PTHREAD_COND_INITIALIZER,
-      -1,
-      SU_MSG_R_INIT,
-      NULL
-    };
-
-    int thread_created = 0;
-    pthread_t tid;
-
-    su_port_threadsafe(parent->sur_port);
-
-    arg.self = child;
-    arg.init = init;
-    arg.deinit = deinit;
-    arg.parent = parent;
-
-    child->sur_magic = magic;
-    child->sur_deinit = deinit;
-    child->sur_threading = parent->sur_threading;
-
-    SU_TASK_COPY(child->sur_parent, su_root_task(parent), su_clone_start);
-
-    pthread_mutex_lock(&arg.mutex);
-    if (pthread_create(&tid, NULL, su_clone_main, &arg) == 0) {
-      pthread_cond_wait(&arg.cv, &arg.mutex);
-      thread_created = 1;
-    }
-    pthread_mutex_unlock(&arg.mutex);
-
-    if (arg.retval != 0) {
-      if (thread_created)
-	pthread_join(tid, NULL);
-      su_root_destroy(child), child = NULL;
-    }
-    else {
-      retval = 0;
-      *return_clone = *arg.clone;
-    }
-  } else
-#endif
-  if (child) {
-    assert(parent);
-
-    child->sur_magic = magic;
-    child->sur_deinit = deinit;
-    child->sur_threading = parent->sur_threading;
-
-    SU_TASK_COPY(child->sur_parent, su_root_task(parent), su_clone_start);
-    SU_TASK_COPY(child->sur_task, child->sur_parent, su_clone_start);
-    su_task_attach(child->sur_task, child);
-
-    if (su_msg_create(return_clone,
-		      child->sur_task, su_root_task(parent),
-		      su_clone_xyzzy, sizeof(child)) == 0) {
-      if (init == NULL || init(child, magic) == 0) {
-	su_cloned_t *sc = su_msg_data(return_clone);
-	sc->sc_root = child;
-#if SU_HAVE_PTHREADS
-	sc->sc_tid = pthread_self();
-	pthread_mutex_init(sc->sc_pause, NULL);
-	pthread_cond_init(sc->sc_resume, NULL);
-	pthread_mutex_lock(sc->sc_pause);
-#endif
-	retval = 0;
-      } else {
-	if (deinit)
-	  deinit(child, magic);
-	su_msg_destroy(return_clone);
-	su_root_destroy(child), child = NULL;
-      }
-    }
-    else {
-      su_root_destroy(child), child = NULL;
-    }
-  }
-
-  return retval;
-}
-
-/** Get reference to clone task.
- * 
- * @param clone Clone pointer
- *
- * @return A reference to the task structure of the clone.
- */
-_su_task_r su_clone_task(su_clone_r clone)
-{
-  return su_msg_to(clone);
-}
-
-/**Forget the clone.
- * 
- * Normally, the clone task executes until it is stopped.  If the parent
- * task does not need to stop the task, it can "forget" the clone.  The
- * clone exits independently of the parent task.
- *
- * @param rclone Reference to the clone.
- */
-void su_clone_forget(su_clone_r rclone)
-{
-  su_msg_destroy(rclone);
-}
-
-/** Stop the clone.
- *
- * @deprecated. Use su_clone_wait().
- */
-void su_clone_stop(su_clone_r rclone)
-{
-  su_msg_send(rclone);
-}
-
-/** Stop a clone and wait until it is has completed.
- *
- * The function su_clone_wait() is used to stop the clone task and wait
- * until it has cleaned up. The clone task is destroyed asynchronously. The
- * parent sends a message to clone, clone deinitializes itself and then
- * replies. After the reply message is received by the parent, it will send
- * a third message back to clone.
- *
- * The parent destroy all messages to or from clone task before calling
- * su_clone_wait(). The parent task may not send any messages to the clone
- * after calling su_clone_wait(). The su_clone_wait() function blocks until
- * the cloned task is destroyed. During that time, the parent task must be
- * prepared to process all the messages sent by clone task. This includes
- * all the messages sent by clone before destroy message reached the clone.
- */
-void su_clone_wait(su_root_t *root, su_clone_r rclone)
-{
-  su_cloned_t *sc = su_msg_data(rclone);
-
-  if (sc) {
-#if SU_HAVE_PTHREADS
-    pthread_t clone_tid = sc->sc_tid;
-#endif
-    int one = 1;
-    /* This does 3-way handshake. 
-     * First, su_clone_break() is executed by clone. 
-     * The message is returned to parent (this task), 
-     * which executes su_clone_report().
-     * Then the message is again returned to clone, 
-     * which executes su_clone_report2() and exits.
-     */
-    sc->sc_wait = &one;
-    su_msg_send(rclone);
-
-    su_root_step(root, 0);
-    su_root_step(root, 0);
-
-    while (one)
-      su_root_step(root, 10);
-
-#if SU_HAVE_PTHREADS
-    if (!pthread_equal(clone_tid, pthread_self()))
-      pthread_join(clone_tid, NULL);
-#endif
-  }
-}
-
-#if SU_HAVE_PTHREADS		/* No-op without threads */
-static
-void su_clone_paused(su_root_magic_t *magic, su_msg_r msg, su_msg_arg_t *arg)
-{
-  su_cloned_t *cloned = *(su_cloned_t **)arg;
-  assert(cloned);
-  pthread_cond_wait(cloned->sc_resume, cloned->sc_pause);
-}
-#endif
-
-/** Pause a clone.
- *
- * Obtain a exclusive lock on clone's private data.
- *
- * @retval 0 if successful (and clone is paused)
- * @retval -1 upon an error
- */
-int su_clone_pause(su_clone_r rclone)
-{
-#if SU_HAVE_PTHREADS		/* No-op without threads */
-  su_cloned_t *cloned = su_msg_data(rclone);
-  su_msg_r m = SU_MSG_R_INIT;
-
-  if (!cloned)
-    return (errno = EFAULT), -1;
-
-  if (pthread_equal(pthread_self(), cloned->sc_tid))
-    return 0;
-
-  if (su_msg_create(m, su_clone_task(rclone), su_task_null,
-		    su_clone_paused, sizeof cloned) < 0)
-    return -1;
-
-  *(su_cloned_t **)su_msg_data(m) = cloned;
-
-  if (su_msg_send(m) < 0)
-    return -1;
-
-  if (pthread_mutex_lock(cloned->sc_pause) < 0)
-    return -1;
-  pthread_cond_signal(cloned->sc_resume);
-#endif
-
-  return 0;
-}
-
-/** Resume a clone.
- *
- * Give up a exclusive lock on clone's private data.
- *
- * @retval 0 if successful (and clone is resumed)
- * @retval -1 upon an error
- */
-int su_clone_resume(su_clone_r rclone)
-{
-#if SU_HAVE_PTHREADS		/* No-op without threads */
-  su_cloned_t *cloned = su_msg_data(rclone);
-
-  if (!cloned)
-    return (errno = EFAULT), -1;
-
-  if (pthread_equal(pthread_self(), cloned->sc_tid))
-    return 0;
-
-  if (pthread_mutex_unlock(cloned->sc_pause) < 0)
-    return -1;
-#endif
-
-  return 0;
-}
-
-
 /* =========================================================================
  * Messages
  */
@@ -1381,16 +852,14 @@ int su_msg_create(su_msg_r        rmsg,
 		  su_task_r const to,
 		  su_task_r const from,
 		  su_msg_f        wakeup,
-		  int             size)
+		  isize_t         size)
 {
-  su_port_t *port = to->sut_port;
   su_msg_t *msg;
 
-  SU_PORT_LOCK(port, su_msg_create);
-  msg = su_salloc(NULL /*port->sup_home*/, sizeof(*msg) + size);
-  SU_PORT_UNLOCK(port, su_msg_create);
+  msg = su_zalloc(NULL, sizeof(*msg) + size);
 
   if (msg) {
+    msg->sum_size = sizeof(*msg) + size;
     SU_TASK_COPY(msg->sum_to, to, su_msg_create);
     SU_TASK_COPY(msg->sum_from, from, su_msg_create);
     msg->sum_func = wakeup;
@@ -1402,8 +871,12 @@ int su_msg_create(su_msg_r        rmsg,
   return -1;
 }
 
-/** Add a report function to a message
- *
+/** Add a delivery report function to a message.
+ * 
+ * The delivery report funcgtion gets called by the sending task after the
+ * message was delivered and the message function was executed. (The
+ * su_root_t message delivery loop calls su_msg_delivery_report() 
+ * 
  */
 int su_msg_report(su_msg_r msg,
 		  su_msg_f report)
@@ -1429,7 +902,7 @@ int su_msg_report(su_msg_r msg,
  */
 
 int su_msg_reply(su_msg_r reply, su_msg_r const msg,
-		 su_msg_f wakeup, int size)
+		 su_msg_f wakeup, isize_t size)
 {
   su_msg_r msg0;
 
@@ -1442,23 +915,35 @@ int su_msg_reply(su_msg_r reply, su_msg_r const msg,
 }
 
 
-/** Send a delivery report 
+/** Send a delivery report.
+ *
+ * If the sender has attached a delivery report function to message with
+ * su_msg_report(), the message is returned to the message queue of the
+ * sending task. The sending task calls the delivery report function when it
+ * has received the message.
  */
 void su_msg_delivery_report(su_msg_r msg)
 {
   su_task_r swap;
 
+  if (!msg || !msg[0])
+    return;
+
+  if (!msg[0]->sum_report) {
+    su_msg_destroy(msg);
+    return;
+  }
+
   *swap = *msg[0]->sum_from;
   *msg[0]->sum_from = *msg[0]->sum_to;
   *msg[0]->sum_to = *swap;
 
   msg[0]->sum_func = msg[0]->sum_report;
   msg[0]->sum_report = NULL;
-  
   su_msg_send(msg);
 }
 
-/** Save a message */
+/** Save a message. */
 void su_msg_save(su_msg_r save, su_msg_r msg)
 {
   if (save) {
@@ -1483,14 +968,14 @@ void su_msg_destroy(su_msg_r rmsg)
   if (rmsg[0]) {
     /* su_port_t *port = rmsg[0]->sum_to->sut_port; */
 
-    /* SU_PORT_INCREF(port, su_msg_destroy); */
+    /* su_port_incref(port, "su_msg_destroy"); */
     SU_TASK_ZAP(rmsg[0]->sum_to, su_msg_destroy);
     SU_TASK_ZAP(rmsg[0]->sum_from, su_msg_destroy);
 
     su_free(NULL /* port->sup_home */, rmsg[0]);
     /* SU_PORT_UNLOCK(port, su_msg_destroy); */
 
-    /* SU_PORT_DECREF(port, su_msg_destroy); */
+    /* su_port_decref(port, "su_msg_destroy"); */
   }
 
   rmsg[0] = NULL;
@@ -1515,7 +1000,7 @@ su_msg_arg_t *su_msg_data(su_msg_cr rmsg)
 }
 
 /** Get size of message data area. */
-int su_msg_size(su_msg_cr rmsg)
+isize_t su_msg_size(su_msg_cr rmsg)
 {
   return rmsg[0] ? rmsg[0]->sum_size - sizeof(su_msg_t) : 0;
 }
