@@ -31,6 +31,7 @@
  * (poll()/select()/WaitForMultipleObjects()) functionality.
  *
  * @author Pekka Pessi <Pekka.Pessi@nokia.com>
+ * @author Martti Mela <Martti.Mela@nokia.com>
  * @date Created: Tue Sep 14 15:51:04 1999 ppessi
  *
  */
@@ -149,7 +150,7 @@ int su_wait_create(su_wait_t *newwait, su_socket_t socket, int events)
 
   *newwait = h;
 
-#elif SU_HAVE_POLL
+#elif SU_HAVE_POLL || HAVE_SELECT
   int mode;
 
   if (newwait == NULL || events == 0 || socket == INVALID_SOCKET) {
@@ -187,7 +188,9 @@ int su_wait_destroy(su_wait_t *waitobj)
   su_wait_t w0 = NULL;
   if (*waitobj)
     WSACloseEvent(*waitobj);
-#elif SU_HAVE_POLL
+#elif SU_HAVE_POLL || HAVE_SELECT
+  su_wait_t w0 = { INVALID_SOCKET, 0, 0 };
+#else
   su_wait_t w0 = { INVALID_SOCKET, 0, 0 };
 #endif
   assert(waitobj != NULL);
@@ -231,7 +234,7 @@ int su_wait(su_wait_t waits[], unsigned n, su_duration_t timeout)
   else
     return i;
 
-#elif SU_HAVE_POLL
+#elif SU_HAVE_POLL || HAVE_SELECT
   for (;;) {
     int i = poll(waits, n, timeout);
 
@@ -249,7 +252,7 @@ int su_wait(su_wait_t waits[], unsigned n, su_duration_t timeout)
     if (errno == EINTR)
       continue;
 
-    return i;
+    return -1;
   }  
 #endif
 }
@@ -273,7 +276,7 @@ int su_wait_events(su_wait_t *waitobj, su_socket_t s)
 
   return net_events.lNetworkEvents;
 
-#elif SU_HAVE_POLL
+#elif SU_HAVE_POLL || HAVE_SELECT
   /* poll(e, 1, 0); */
   return waitobj->revents;
 #endif
@@ -302,7 +305,8 @@ int su_wait_mask(su_wait_t *waitobj, su_socket_t s, int events)
     WSASetLastError(error);
     return -1;
   }
-#elif SU_HAVE_POLL
+
+#elif SU_HAVE_POLL || HAVE_SELECT
   waitobj->fd = s;
   waitobj->events = events;
   waitobj->revents = 0;
