@@ -90,14 +90,24 @@ su_log_t tport_log[] = {
 /** Initialize logging. */
 void tport_open_log(tport_master_t *mr, tagi_t *tags)
 {
-  char const *log;
-  
-  mr->mr_log = 
-    getenv("MSG_STREAM_LOG") != NULL ||
-    getenv("TPORT_LOG") != NULL 
-    ? MSG_DO_EXTRACT_COPY : 0;
-    
-  if ((log = getenv("TPORT_DUMP")) || (log = getenv("MSG_DUMP"))) {
+  char const *log = NULL;
+  int log_msg = 0;
+
+  tl_gets(tags, TPTAG_LOG_REF(log_msg), TAG_END());
+
+  if (getenv("MSG_STREAM_LOG") != NULL || getenv("TPORT_LOG") != NULL)
+    log_msg = 1;
+
+  mr->mr_log = log_msg ? MSG_DO_EXTRACT_COPY : 0;
+
+  tl_gets(tags, TPTAG_DUMP_REF(log), TAG_END());
+
+  if (getenv("MSG_DUMP"))
+    log = getenv("MSG_DUMP");
+  if (getenv("TPORT_DUMP"))
+    log = getenv("TPORT_DUMP");
+
+  if (log) {
     time_t now;
 
     if (strcmp(log, "-")) 
@@ -166,7 +176,8 @@ void tport_dump_iovec(tport_t const *self, msg_t *msg,
     size_t len = iov[i].mv_len;
     if (len > n)
       len = n;
-    fwrite(iov[i].mv_base, len, 1, mr->mr_dump_file);
+    if (fwrite(iov[i].mv_base, len, 1, mr->mr_dump_file) != len)
+      break;
     n -= len;
   }
 
@@ -210,17 +221,17 @@ void tport_log_msg(tport_t *self, msg_t *msg,
       n = strncspn(s, end - s, "\r\n");
 
       if (linelen + n > MAX_LINELEN) {
-	n = MAX_LINELEN - n - linelen;
+	n = MAX_LINELEN - linelen;
 	truncated = logged + n;
       }
 
-      su_log("%s%.*s", linelen > n ? "" : "   ", (int)n, s);
+      su_log("%s%.*s", linelen > 0 ? "" : "   ", (int)n, s);
       s += n, linelen += n, logged += n;
 
       if (truncated)
 	break;
       if (s == end)
-	continue;
+	break;
 
       linelen = 0;
       su_log("\n");
