@@ -37,8 +37,6 @@
 
 #include "config.h"
 
-#if HAVE_SCTP
-
 #include "tport_internal.h"
 
 #if HAVE_NETINET_SCTP_H
@@ -59,6 +57,12 @@
 #undef MAX_STREAMS
 #define MAX_STREAMS MAX_STREAMS
 
+/* Missing socket symbols */
+#ifndef SOL_SCTP
+#define SOL_SCTP IPPROTO_SCTP
+#endif
+
+
 enum { MAX_STREAMS = 1 };
 typedef struct tport_sctp_t
 {
@@ -76,6 +80,8 @@ typedef struct tport_sctp_t
 
 #define TP_SCTP_MSG_MAX (65536)
 
+#if HAVE_SCTP
+
 static int tport_sctp_init_primary(tport_primary_t *, 
 				   tp_name_t tpn[1], 
 				   su_addrinfo_t *, tagi_t const *,
@@ -90,8 +96,8 @@ static int tport_sctp_init_socket(tport_primary_t *pri,
 				  int socket,
 				  char const **return_reason);
 static int tport_recv_sctp(tport_t *self);
-static int tport_send_sctp(tport_t const *self, msg_t *msg,
-			   msg_iovec_t iov[], int iovused);
+static ssize_t tport_send_sctp(tport_t const *self, msg_t *msg,
+			       msg_iovec_t iov[], size_t iovused);
 
 tport_vtable_t const tport_sctp_client_vtable =
 {
@@ -172,10 +178,7 @@ static int tport_sctp_init_client(tport_primary_t *pri,
 static int tport_sctp_init_secondary(tport_t *self, int socket, int accepted,
 				     char const **return_reason)
 {
-  self->tp_connected = 1;
-
-  if (su_setblocking(socket, 0) < 0)
-    return *return_reason = "su_setblocking", -1;
+  self->tp_has_connection = 1;
 
   if (accepted) {
     /* Accepted socket inherit the init information from listen socket */
@@ -251,8 +254,8 @@ int tport_recv_sctp(tport_t *self)
   return 2;
 }
 
-static int tport_send_sctp(tport_t const *self, msg_t *msg,
-			   msg_iovec_t iov[], int iovused)
+static ssize_t tport_send_sctp(tport_t const *self, msg_t *msg,
+			       msg_iovec_t iov[], size_t iovused)
 {
   
 
