@@ -49,11 +49,6 @@ static int tstflags = 0;
 
 char const name[] = "torture_url";
 
-void usage(void)
-{
-  fprintf(stderr, "usage: %s [-v]\n", name);
-}
-
 unsigned char hash1[16], hash2[16];
 
 /* test unquoting and canonizing */
@@ -81,6 +76,9 @@ int test_quote(void)
   
   BEGIN();
 
+  d = url_as_string(home, (url_t *)"sip:joe@example.com");
+  TEST_S(d, "sip:joe@example.com");
+
   TEST(strlen(EXCLUDED), 23);
   TEST(strlen(UNRESERVED), 71);
 
@@ -88,12 +86,12 @@ int test_quote(void)
   TEST_1(!url_reserved_p(""));
   TEST_1(url_reserved_p("foobar:bar"));
 
-  TEST(url_esclen("a" EXCLUDED, ""), 
-       1 + strlen(RESERVED) + 3 * strlen(DELIMS UNWISE));
-  TEST(url_esclen("a" EXCLUDED, DELIMS UNWISE), 
-       1 + strlen(RESERVED) + 3 * strlen(DELIMS UNWISE));
-  TEST(url_esclen("a" EXCLUDED, EXCLUDED), 1 + 3 * strlen(EXCLUDED));
-  TEST(url_esclen("a" EXCLUDED, NULL), 1 + 3 * strlen(EXCLUDED));
+  TEST_SIZE(url_esclen("a" EXCLUDED, ""), 
+	    1 + strlen(RESERVED) + 3 * strlen(DELIMS UNWISE));
+  TEST_SIZE(url_esclen("a" EXCLUDED, DELIMS UNWISE), 
+	    1 + strlen(RESERVED) + 3 * strlen(DELIMS UNWISE));
+  TEST_SIZE(url_esclen("a" EXCLUDED, EXCLUDED), 1 + 3 * strlen(EXCLUDED));
+  TEST_SIZE(url_esclen("a" EXCLUDED, NULL), 1 + 3 * strlen(EXCLUDED));
 
   TEST_S(url_escape(escaped, "a" EXCLUDED, NULL),
 	 "a%3B%2F%3F%3A%40%26%3D%2B%24%2C"
@@ -101,11 +99,11 @@ int test_quote(void)
 	 "%7B%7D%7C%5C%5E%5B%5D%60");
   TEST_S(url_unescape(escaped, escaped), "a" EXCLUDED);
 
-  TEST(url_esclen(UNRESERVED, NULL), strlen(UNRESERVED));
+  TEST_SIZE(url_esclen(UNRESERVED, NULL), strlen(UNRESERVED));
   TEST_S(url_escape(unreserved, UNRESERVED, NULL), UNRESERVED);
   TEST_S(url_unescape(unreserved, UNRESERVED), UNRESERVED);
 
-  d = "%73ip:%55@%68";
+  d = "%53ip:%75@%48";		/* Sip:u@H */
   u = url_hdup(home, (url_t *)d); TEST_1(u);
   url_digest(hash1, sizeof(hash1), u, NULL);
   url_digest(hash2, sizeof(hash2), (url_t const *)d, NULL);
@@ -122,9 +120,11 @@ int test_quote(void)
   d = url_as_string(home, u); TEST_1(d);
   TEST_S(d, c);
 
-  d = "sip:&=+$,;?/:&=+$,@host:56001;param=+$,/:@&;another=@"
+  d = "sip:&=+$,;?/:&=+$,@[::1]:56001;param=+$,/:@&;another=@"
     "?header=" RESERVED "&%3b%2f%3f%3a%40%26%3d%2b%24%2c";
   u = url_hdup(home, (url_t *)d); TEST_1(u);
+  TEST_S(u->url_user, "&=+$,;?/");
+  TEST_S(u->url_host, "[::1]");
   TEST_S(u->url_headers, "header=" RESERVED "&%3B%2F%3F%3A%40%26%3D%2B%24%2C");
   url_digest(hash1, sizeof(hash1), u, NULL);
   url_digest(hash2, sizeof(hash2), (url_t const *)d, NULL);
@@ -134,8 +134,10 @@ int test_quote(void)
   d = url_as_string(home, u); TEST_1(d);
   TEST_S(d, c);
 
-  d = "http://&=+$,;:&=+$,;@host:8080/foo;param=+$,/:@&;another=@?query=" RESERVED;
+  d = "http://&=+$,;:&=+$,;@host:8080/foo;param=+$,/:@&;another=@"
+    "?query=" RESERVED;
   u = url_hdup(home, (url_t *)d); TEST_1(u);
+  TEST_S(u->url_user, "&=+$,;"); TEST_S(u->url_password, "&=+$,;");
   url_digest(hash1, sizeof(hash1), u, NULL);
   url_digest(hash2, sizeof(hash2), (url_t const *)d, NULL);
   TEST(memcmp(hash1, hash2, sizeof(hash1)), 0);
@@ -170,6 +172,7 @@ int test_any(void)
   TEST_S(url_scheme(url_im), "im");
   TEST_S(url_scheme(url_cid), "cid");
   TEST_S(url_scheme(url_msrp), "msrp");
+  TEST_S(url_scheme(url_msrps), "msrps");
 
   TEST_1(tst = su_strdup(home, "*"));
   TEST(url_d(url, tst), 0);
@@ -187,8 +190,8 @@ int test_any(void)
     char buf[6];
     
     TEST_1(u = url_hdup(home, (void *)"error"));
-    TEST(url_xtra(u), 6);
-    TEST(url_dup(buf, 6, url, u), 6);
+    TEST_SIZE(url_xtra(u), 6);
+    TEST_SIZE(url_dup(buf, 6, url, u), 6);
     TEST_S(buf, "error");
   }
 
@@ -236,7 +239,7 @@ int test_sip(void)
   char sipsurl[] = 
     "sips:user:pass@host:32;param=1"
     "?From=foo@bar&To=bar@baz#unf";
-  int i, j;
+  size_t i, j;
   url_t *a, *b;
 
   BEGIN();
@@ -252,13 +255,16 @@ int test_sip(void)
   TEST(url->url_root, 0);
   TEST_S(url->url_scheme, "sip");
   TEST_S(url->url_user, "pekka.pessi");
-  TEST(url->url_password, NULL);
+  TEST_P(url->url_password, NULL);
   TEST_S(url->url_host, "nokia.com");
-  TEST(url->url_port, NULL);
-  TEST(url->url_path, NULL);
+  TEST_P(url->url_port, NULL);
+  TEST_P(url->url_path, NULL);
   TEST_S(url->url_params, "method=MESSAGE");
   TEST_S(url->url_headers, "body=CANNED%20MSG");
-  TEST(url->url_fragment, NULL);
+  TEST_P(url->url_fragment, NULL);
+
+  TEST_S(url_query_as_header_string(home, url->url_headers),
+	 "\n\nCANNED MSG");
   
   sip->url_user = "user";
   sip->url_password = "pass";
@@ -277,10 +283,10 @@ int test_sip(void)
   TEST_1(u = url_hdup(home, url));
   TEST(u->url_type, url_sip);
   TEST_1(url_cmp(sip, u) == 0);
-  TEST_1(url_e(sip2, sizeof(sip2), u) == strlen(sipurl));
+  TEST(url_e(sip2, sizeof(sip2), u), strlen(sipurl));
   TEST_1(strcmp(sip2, sipurl) == 0);
-  TEST(snprintf(sip2, sizeof(sip2), URL_PRINT_FORMAT, 
-		URL_PRINT_ARGS(sip)), strlen(sipurl));
+  TEST_SIZE(snprintf(sip2, sizeof(sip2), URL_PRINT_FORMAT, 
+		     URL_PRINT_ARGS(sip)), strlen(sipurl));
   TEST_1(strcmp(sip2, sipurl) == 0);
 
   url_digest(hash1, sizeof(hash1), url, NULL);
@@ -291,6 +297,9 @@ int test_sip(void)
   TEST_1(url_d(url, tst) == 0);
   TEST_S(url->url_user, "user/path;tel-param");
 
+  TEST_S(url_query_as_header_string(home, url->url_headers),
+	 "From:foo@bar\nTo:bar@baz");
+
   url_digest(hash1, sizeof(hash1), url, NULL);
   url_digest(hash2, sizeof(hash2), (url_t *)sip2url, NULL);
   TEST(memcmp(hash1, hash2, sizeof(hash1)), 0);
@@ -308,7 +317,7 @@ int test_sip(void)
     url_t dst[1];
 
     buf[i] = '\377';
-    TEST(url_dup(buf, i, dst, url), sizeof(sipsurl) - 1 - strlen("sips"));
+    TEST_SIZE(url_dup(buf, i, dst, url), sizeof(sipsurl) - 1 - strlen("sips"));
     TEST(buf[i], '\377');
   }
 
@@ -319,9 +328,16 @@ int test_sip(void)
   u = url_hdup(home, (url_t*)"SIP:test@127.0.0.1:55"); TEST_1(u);
   TEST(u->url_type, url_sip);
 
-  TEST(url_hdup(home, (url_t*)"sip:test@127.0.0.1::55"), NULL);
-  TEST(url_hdup(home, (url_t*)"sip:test@127.0.0.1:55:"), NULL);
-  TEST(url_hdup(home, (url_t*)"sip:test@127.0.0.1:sip"), NULL);
+  TEST_P(url_hdup(home, (url_t*)"sip:test@127.0.0.1::55"), NULL);
+  TEST_P(url_hdup(home, (url_t*)"sip:test@127.0.0.1:55:"), NULL);
+  TEST_P(url_hdup(home, (url_t*)"sip:test@127.0.0.1:sip"), NULL);
+
+  u = url_hdup(home, (url_t*)"SIP:#**00**#;foo=/bar@127.0.0.1"); TEST_1(u);
+  TEST(u->url_type, url_sip);
+  TEST_S(u->url_user, "#**00**#;foo=/bar");
+
+  TEST_1(!url_hdup(home, (url_t*)"SIP:#**00**#;foo=/bar@#127.0.0.1"));
+  TEST_1(!url_hdup(home, (url_t*)"SIP:#**00**#;foo=/bar;127.0.0.1"));
 
   for (i = 32; i <= 256; i++) {
     char pu[512];
@@ -338,11 +354,16 @@ int test_sip(void)
     TEST_S(pu, s);
   }
 
+  s = su_strdup(home, "ttl;transport=tcp;ttl=15;ttl=;method=INVITE;ttl");
+  TEST_1(s);
+  s = url_strip_param_string(s, "ttl");
+  TEST_S(s, "transport=tcp;method=INVITE");
+
   u = url_hdup(home, (void*)"sip:u:p@host:5060;maddr=127.0.0.1;transport=tcp");
   TEST_1(u);
   TEST_1(url_have_transport(u));
   TEST_1(url_strip_transport(u));
-  TEST(u->url_params, NULL);
+  TEST_P(u->url_params, NULL);
   TEST_1(!url_have_transport(u));
 
   u = url_hdup(home, (void*)"sip:u:p@host:5060;user=phone;ttl=1;isfocus");
@@ -382,6 +403,10 @@ int test_sip(void)
   TEST_S(u->url_params, "isfocus");
   TEST_1(!url_have_transport(u));
 
+  u = url_hdup(home, (void *)"sip:%22foo%22@172.21.55.55:5060");
+  TEST_1(u);
+  TEST_S(u->url_user, "%22foo%22");
+
   a = url_hdup(home, (void *)"sip:172.21.55.55:5060");
   b = url_hdup(home, (void *)"sip:172.21.55.55");
   TEST_1(a); TEST_1(b);
@@ -448,10 +473,10 @@ int test_wv(void)
   TEST_1(u = url_hdup(home, url));
   TEST(u->url_type, url_wv);
   TEST_1(url_cmp(wv, u) == 0);
-  TEST(url_e(wv2, sizeof(wv2), u), strlen(wvurl));
+  TEST_SIZE(url_e(wv2, sizeof(wv2), u), strlen(wvurl));
   TEST_1(strcmp(wv2, wvurl) == 0);
-  TEST(snprintf(wv2, sizeof(wv2), URL_PRINT_FORMAT, 
-		URL_PRINT_ARGS(wv)), strlen(wvurl));
+  TEST_SIZE(snprintf(wv2, sizeof(wv2), URL_PRINT_FORMAT, 
+		     URL_PRINT_ARGS(wv)), strlen(wvurl));
   TEST_1(strcmp(wv2, wvurl) == 0);
 
   url_digest(hash1, sizeof(hash1), url, NULL);
@@ -500,10 +525,10 @@ int test_tel(void)
   TEST_1(u = url_hdup(home, url));
   TEST(u->url_type, url_tel);
   TEST_1(url_cmp(tel, u) == 0);
-  TEST(url_e(tel2, sizeof(tel2), u), strlen(telurl));
+  TEST_SIZE(url_e(tel2, sizeof(tel2), u), strlen(telurl));
   TEST_1(strcmp(tel2, telurl) == 0);
-  TEST(snprintf(tel2, sizeof(tel2), URL_PRINT_FORMAT, 
-		URL_PRINT_ARGS(tel)), strlen(telurl));
+  TEST_SIZE(snprintf(tel2, sizeof(tel2), URL_PRINT_FORMAT, 
+		     URL_PRINT_ARGS(tel)), strlen(telurl));
   TEST_1(strcmp(tel2, telurl) == 0);
 
   url_digest(hash1, sizeof(hash1), url, NULL);
@@ -550,10 +575,10 @@ int test_fax(void)
   TEST_1(u = url_hdup(home, url));
   TEST(u->url_type, url_fax);
   TEST_1(url_cmp(fax, u) == 0);
-  TEST(url_e(fax2, sizeof(fax2), u), strlen(faxurl));
+  TEST_SIZE(url_e(fax2, sizeof(fax2), u), strlen(faxurl));
   TEST_1(strcmp(fax2, faxurl) == 0);
-  TEST(snprintf(fax2, sizeof(fax2), URL_PRINT_FORMAT, 
-		URL_PRINT_ARGS(fax)), strlen(faxurl));
+  TEST_SIZE(snprintf(fax2, sizeof(fax2), URL_PRINT_FORMAT, 
+		     URL_PRINT_ARGS(fax)), strlen(faxurl));
   TEST_1(strcmp(fax2, faxurl) == 0);
 
 
@@ -595,10 +620,10 @@ int test_modem(void)
   TEST_1(u = url_hdup(home, url));
   TEST(u->url_type, url_modem);
   TEST_1(url_cmp(modem, u) == 0);
-  TEST(url_e(modem2, sizeof(modem2), u), strlen(modemurl));
+  TEST_SIZE(url_e(modem2, sizeof(modem2), u), strlen(modemurl));
   TEST_1(strcmp(modem2, modemurl) == 0);
-  TEST(snprintf(modem2, sizeof(modem2), URL_PRINT_FORMAT, 
-		URL_PRINT_ARGS(modem)), strlen(modemurl));
+  TEST_SIZE(snprintf(modem2, sizeof(modem2), URL_PRINT_FORMAT, 
+		     URL_PRINT_ARGS(modem)), strlen(modemurl));
   TEST_1(strcmp(modem2, modemurl) == 0);
 
   url_digest(hash1, sizeof(hash1), url, NULL);
@@ -613,7 +638,7 @@ int test_modem(void)
 int test_file(void)
 {
   /* Test a url with path like file:/foo/bar  */
-  char fileurl[] = "file:/foo/bar";
+  char fileurl[] = "file:///foo/bar";
   url_t file[1] = { URL_INIT_AS(file) };
   su_home_t home[1] = { SU_HOME_INIT(home) };
   char *tst;
@@ -627,17 +652,19 @@ int test_file(void)
 
   TEST_1(tst = su_strdup(home, fileurl));
   TEST(url_d(url, tst), 0);
+  TEST_S(url->url_host, "");
   file->url_root = '/';
+  file->url_host = "";
   file->url_path = "foo/bar";
   TEST(url_cmp(file, url), 0);
   TEST(url->url_type, url_file);
   TEST_1(u = url_hdup(home, url));
   TEST(u->url_type, url_file);
   TEST(url_cmp(file, u), 0);
-  TEST(url_e(buf1, sizeof(buf1), u), strlen(fileurl));
+  TEST_SIZE(url_e(buf1, sizeof(buf1), u), strlen(fileurl));
   TEST_S(buf1, fileurl);
-  TEST(snprintf(buf2, sizeof(buf2), URL_PRINT_FORMAT, URL_PRINT_ARGS(u)), 
-       strlen(fileurl));
+  TEST_SIZE(snprintf(buf2, sizeof(buf2), URL_PRINT_FORMAT, URL_PRINT_ARGS(u)), 
+	    strlen(fileurl));
   TEST_S(buf2, fileurl);
 
   url_digest(hash1, sizeof(hash1), url, NULL);
@@ -690,10 +717,10 @@ int test_ldap(void)
   TEST(u->url_type, url_unknown);
   TEST_S(u->url_scheme, "ldap");
   TEST(url_cmp(ldap, u), 0);
-  TEST(url_e(buf1, sizeof(buf1), u), strlen(ldapurl));
+  TEST_SIZE(url_e(buf1, sizeof(buf1), u), strlen(ldapurl));
   TEST_S(buf1, ldapurl);
-  TEST(snprintf(buf2, sizeof(buf2), URL_PRINT_FORMAT, URL_PRINT_ARGS(u)), 
-       strlen(ldapurl));
+  TEST_SIZE(snprintf(buf2, sizeof(buf2), URL_PRINT_FORMAT, URL_PRINT_ARGS(u)), 
+	    strlen(ldapurl));
   TEST_S(buf2, ldapurl);
 
   url_digest(hash1, sizeof(hash1), url, NULL);
@@ -733,10 +760,10 @@ int test_rtsp(void)
   TEST_1(u = url_hdup(home, url));
   TEST(u->url_type, url_rtsp);
   TEST_1(url_cmp(rtsp, u) == 0);
-  TEST_1(url_e(rtsp2, sizeof(rtsp2), u) == strlen(rtspurl));
+  TEST(url_e(rtsp2, sizeof(rtsp2), u), strlen(rtspurl));
   TEST_1(strcmp(rtsp2, rtspurl) == 0);
-  TEST(snprintf(rtsp2, sizeof(rtsp2), URL_PRINT_FORMAT, 
-		URL_PRINT_ARGS(rtsp)), strlen(rtspurl));
+  TEST_SIZE(snprintf(rtsp2, sizeof(rtsp2), URL_PRINT_FORMAT, 
+		     URL_PRINT_ARGS(rtsp)), strlen(rtspurl));
   TEST_1(strcmp(rtsp2, rtspurl) == 0);
 
   url_digest(hash1, sizeof(hash1), url, NULL);
@@ -796,10 +823,10 @@ int test_http(void)
   TEST_1(u = url_hdup(home, url));
   TEST(u->url_type, url_http);
   TEST_1(url_cmp(http, u) == 0);
-  TEST(url_e(http2, sizeof(http2), u), strlen(httpurl));
+  TEST_SIZE(url_e(http2, sizeof(http2), u), strlen(httpurl));
   TEST_1(strcmp(http2, httpurl) == 0);
-  TEST(snprintf(http2, sizeof(http2), URL_PRINT_FORMAT, 
-		URL_PRINT_ARGS(http)), strlen(httpurl));
+  TEST_SIZE(snprintf(http2, sizeof(http2), URL_PRINT_FORMAT, 
+		     URL_PRINT_ARGS(http)), strlen(httpurl));
   TEST_1(strcmp(http2, httpurl) == 0);
 
   url_digest(hash1, sizeof(hash1), http, NULL);
@@ -812,6 +839,16 @@ int test_http(void)
   TEST_S(url->url_host, "some.host");
   TEST_S(url->url_headers, "query");
   TEST_S(url->url_params, NULL);
+
+  TEST_1(u = url_hdup(home, (void *)"http://[::1]/test;ing?here"));
+  TEST_S(u->url_host, "[::1]");
+  TEST_S(u->url_path, "test;ing");
+  TEST_S(u->url_headers, "here");
+
+  url_digest(hash1, sizeof(hash1), u, NULL);
+  url_digest(hash2, sizeof(hash2), (url_t *)"http://[::1]/test;ing?here", 
+	     NULL);
+  TEST(memcmp(hash1, hash2, sizeof(hash1)), 0);
     
   su_home_deinit(home);
 
@@ -870,11 +907,19 @@ int test_tags(void)
   url_string_t *us0 = NULL;
 
   tagi_t *lst, *dup;
-
+  
+  tag_value_t value;
+  char *s;
   su_home_t home[1] = { SU_HOME_INIT(home) };
 
   BEGIN();
 
+  TEST(t_scan(urltag_url, home, c0, &value), 0);
+  TEST_S(s = url_as_string(home, (url_t *)value), c0);
+
+  TEST(t_scan(urltag_url, home, c3, &value), 0);
+  TEST_S(s = url_as_string(home, (url_t *)value), c3);
+
   TEST_1(url_d(u0, c0) == 0);
 
   lst = tl_list(URLTAG_URL(u0),
@@ -930,8 +975,8 @@ int test_tag_filter(void)
   result = tl_afilter(NULL, filter, lst);
 
   TEST_1(result);
-  TEST(result[0].t_tag, urltag_url);
-  TEST(result[1].t_tag, urltag_url);
+  TEST_P(result[0].t_tag, urltag_url);
+  TEST_P(result[1].t_tag, urltag_url);
 
   tl_vfree(lst);
   free(result);
@@ -1003,7 +1048,11 @@ int test_print(void)
   END();
 }
 
-
+void usage(int exitcode)
+{
+  fprintf(stderr, "usage: %s [-v] [-a]\n", name);
+  exit(exitcode);
+}
 
 int main(int argc, char *argv[])
 {
@@ -1013,8 +1062,10 @@ int main(int argc, char *argv[])
   for (i = 1; argv[i]; i++) {
     if (strcmp(argv[i], "-v") == 0)
       tstflags |= tst_verbatim;
+    else if (strcmp(argv[i], "-a") == 0)
+      tstflags |= tst_abort;
     else
-      usage();
+      usage(1);
   }
 
   retval |= test_quote(); fflush(stdout);
